using System;
using Avanzis.QueryFramework;
using NUnit.Framework;

namespace Avanzis.QueryBuilder.NUnit
{
	/// <summary>
	/// Test SqlField
	/// </summary>
	[TestFixture]
	public class SqlFieldTest
	{
        [TestFixtureSetUp]
        public void Setup()
        {
            QueryFrameworkConfiguration.Dialect = Dialects.MSSql2005Dialect;
            QueryBuilder.QueryProcessor.ResetProcessor();
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void NullFieldName()
        {
            SqlField field = new SqlField();
            field.ToString();            
        }

		[Test]
		public void SimpleField()
		{
			SqlField field = new SqlField("CustomerId");
			Assert.AreEqual("CustomerId", field.ToString());

			field = new SqlField("CustomerId", "Id");
			Assert.AreEqual("CustomerId AS Id", field.ToString());

			field = new SqlField("CustomerId", "Id", "Customers");			
			Assert.AreEqual("Customers.CustomerId AS Id", field.ToString());

			field = new SqlField("CustomerId", null, "Customers");
			Assert.AreEqual("Customers.CustomerId", field.ToString());																  

		}

        [Test]
        public void NormalFieldAssigment()
        {
            // El nombre de la tabla y la columna
            SqlField field = new SqlField("Customers.CustomerId");
            Assert.AreEqual("Customers", field.TableName);
            Assert.AreEqual("CustomerId", field.ColumnNameOrExpression);
            Assert.AreEqual(null, field.Alias);

            // El nombre de la tabla en corchetes y la columna
            field = new SqlField("[Customers].CustomerId");
            Assert.AreEqual("[Customers]", field.TableName);
            Assert.AreEqual("CustomerId", field.ColumnNameOrExpression);
            Assert.AreEqual(null, field.Alias);

            // El nombre de la tabla y la columna en corchetes
            field = new SqlField("Customers.[CustomerId]");
            Assert.AreEqual("Customers", field.TableName);
            Assert.AreEqual("[CustomerId]", field.ColumnNameOrExpression);
            Assert.AreEqual(null, field.Alias);

            // El nombre de la tabla, la columna y el Alias
            field = new SqlField("Customers.CustomerId AS cid");
            Assert.AreEqual("Customers", field.TableName);
            Assert.AreEqual("CustomerId", field.ColumnNameOrExpression);
            Assert.AreEqual("cid", field.Alias);

            // El nombre de la tabla, la columna y el Alias entre comillas
            field = new SqlField("Customers.CustomerId AS 'cid'");
            Assert.AreEqual("Customers", field.TableName);
            Assert.AreEqual("CustomerId", field.ColumnNameOrExpression);
            Assert.AreEqual("'cid'", field.Alias);
        }

        [Test]
        public void ExpressionFieldAssigment()
        {
            // Una expresin
            SqlField field = new SqlField("SUM(Quantity)");
            Assert.AreEqual(null, field.TableName);
            Assert.AreEqual("SUM(Quantity)", field.ColumnNameOrExpression);
            Assert.AreEqual(null, field.Alias);

            // Una expresin con Alias
            field = new SqlField("SUM(Quantity) AS Suma");
            Assert.AreEqual(null, field.TableName);
            Assert.AreEqual("SUM(Quantity)", field.ColumnNameOrExpression);
            Assert.AreEqual("Suma", field.Alias);

            // Una expresin con Alias entre comillas simples
            field = new SqlField("SUM(Quantity) AS 'Suma Total'");
            Assert.AreEqual(null, field.TableName);
            Assert.AreEqual("SUM(Quantity)", field.ColumnNameOrExpression);
            Assert.AreEqual("'Suma Total'", field.Alias);

            // Una expresin con Alias y el AS sin que sea mayscula
            field = new SqlField("SUM(Quantity) aS Suma");
            Assert.AreEqual(null, field.TableName);
            Assert.AreEqual("SUM(Quantity)", field.ColumnNameOrExpression);
            Assert.AreEqual("Suma", field.Alias);

            // Una expresin entre parntesis con Alias
            field = new SqlField("(SELECT SUM(Quantity) AS QuantitySum FROM WarehouseMovements) AS Suma");
            Assert.AreEqual(null, field.TableName);
            Assert.AreEqual("(SELECT SUM(Quantity) AS QuantitySum FROM WarehouseMovements)", field.ColumnNameOrExpression);
            Assert.AreEqual("Suma", field.Alias);	
        }

		[Test]
        public void CaseFieldAssigment()
		{
            // Una expresin con CASE en formato bsqueda sin Alias
            SqlField field = new SqlField("CASE WHEN Quantity > 0 THEN 1 ELSE 0");
            Assert.AreEqual(null, field.TableName);
            Assert.AreEqual("CASE WHEN Quantity > 0 THEN 1 ELSE 0", field.ColumnNameOrExpression);
            Assert.AreEqual(null, field.Alias);
            Assert.AreEqual("(CASE WHEN Quantity > 0 THEN 1 ELSE 0)", field.ToString());

            // Una expresin con CASE en formato bsqueda con Alias
            field = new SqlField("CASE WHEN Quantity > 0 THEN 1 ELSE 0 AS HasStock");
            Assert.AreEqual(null, field.TableName);
            Assert.AreEqual("CASE WHEN Quantity > 0 THEN 1 ELSE 0", field.ColumnNameOrExpression);
            Assert.AreEqual("HasStock", field.Alias);            

            // Una expresin con CASE en formato simple sin Alias
            field = new SqlField("CASE SEX WHEN 0 THEN 'Female' WHEN 1 THEN 'Male' WHEN 2 THEN 'Unknown' ELSE 'Invalid Sex'");
            Assert.AreEqual(null, field.TableName);
            Assert.AreEqual("CASE SEX WHEN 0 THEN 'Female' WHEN 1 THEN 'Male' WHEN 2 THEN 'Unknown' ELSE 'Invalid Sex'", field.ColumnNameOrExpression);
            Assert.AreEqual(null, field.Alias);            

            // Una expresin con CASE en formato simple con Alias
            field = new SqlField("CASE SEX WHEN 0 THEN 'Female' WHEN 1 THEN 'Male' WHEN 2 THEN 'Unknown' ELSE 'Invalid Sex' AS PatientSex");
            Assert.AreEqual(null, field.TableName);
            Assert.AreEqual("CASE SEX WHEN 0 THEN 'Female' WHEN 1 THEN 'Male' WHEN 2 THEN 'Unknown' ELSE 'Invalid Sex'", field.ColumnNameOrExpression);
            Assert.AreEqual("PatientSex", field.Alias);
		}


		[Test]
		public void FieldCollection()
		{
			SqlFieldCollection fieldCollection = new SqlFieldCollection();
			fieldCollection.Add(new SqlField("CustomerId"));
			fieldCollection.Add(new SqlField("Name", "Customer Name"));
			fieldCollection.Add(new SqlField("SUM(Sales.Ammount)", "Sales"));
			Assert.AreEqual("CustomerId,Name AS 'Customer Name',SUM(Sales.Ammount) AS Sales", fieldCollection.ToString());
		}
		
	}
}
