using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Expresin de comparacin
    /// </summary>
    public class CompareExpression : QueryExpression
    {
        private SqlOperator compareOperation = SqlOperator.Equals;
        private string leftExpression;
        private string rightExpression;

        /// <summary>
        /// Initializes a new instance of the <see cref="CompareExpression"/> class.
        /// </summary>
        /// <param name="leftExpression">The left expression.</param>
        /// <param name="compareOperation">The compare operation.</param>
        /// <param name="rightExpression">The right expression.</param>
        public CompareExpression(object leftExpression, SqlOperator compareOperation, object rightExpression) : this(leftExpression, compareOperation, rightExpression, ExpressionType.Complex)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:CompareExpression"/> class.
        /// </summary>
        /// <param name="leftExpression">The left expression.</param>
        /// <param name="compareOperation">The compare operation.</param>
        /// <param name="rightExpression">The right expression.</param>
        /// <param name="rightExpressionType">Type of the right expression.</param>
        public CompareExpression(object leftExpression, SqlOperator compareOperation, object rightExpression, ExpressionType rightExpressionType)
        {
            if (leftExpression != null)
                this.leftExpression = QueryProcessor.Current.GetExpressionStringValue(leftExpression);

            this.compareOperation = compareOperation;

            if (QueryProcessor.Current.IsUnaryOperator(compareOperation))
                return;

            if (rightExpressionType == ExpressionType.Unknown)
            {

                ExpressionType operatorExpressionType = QueryProcessor.Current.GetOperatorExpressionType(compareOperation);

                if (operatorExpressionType == ExpressionType.Unknown || operatorExpressionType == ExpressionType.Primitive)
                {
                    String stringValue = rightExpression as String;
                    if (stringValue != null)
                        rightExpressionType = QueryProcessor.Current.GetBestMatchedExpressionType(stringValue);

                    if (rightExpressionType == ExpressionType.Unknown)
                        rightExpressionType = operatorExpressionType;
                }
            }

            this.rightExpression = QueryProcessor.Current.GetExpressionStringValue(rightExpression, rightExpressionType);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CompareExpression"/> class.
        /// </summary>
        /// <param name="compareOperation">The compare operation.</param>
        /// <param name="rightExpression">The right expression.</param>
        public CompareExpression(SqlOperator compareOperation, object rightExpression) : this(compareOperation, rightExpression, ExpressionType.Complex)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:CompareExpression"/> class.
        /// </summary>
        /// <param name="compareOperation">The compare operation.</param>
        /// <param name="rightExpression">The right expression.</param>
        /// <param name="rightExpressionType">Type of the right expression.</param>
        public CompareExpression(SqlOperator compareOperation, object rightExpression, ExpressionType rightExpressionType)
        {
            this.compareOperation = compareOperation;
            this.rightExpression = QueryProcessor.Current.GetExpressionStringValue(rightExpression, rightExpressionType);
        }

        /// <summary>
        /// Gets or sets the left expression.
        /// </summary>
        /// <value>The left expression.</value>
        public string LeftExpression
        {
            get { return leftExpression; }
            set { leftExpression = value; }
        }

        /// <summary>
        /// Gets or sets the right expression.
        /// </summary>
        /// <value>The right expression.</value>
        public string RightExpression
        {
            get { return rightExpression; }
            set { rightExpression = value; }
        }

        /// <summary>
        /// Gets or sets the compare operation.
        /// </summary>
        /// <value>The compare operation.</value>
        public SqlOperator CompareOperation
        {
            get { return compareOperation; }
            set { compareOperation = value; }
        }

        /// <summary>
        /// Gets the expression.
        /// </summary>
        /// <returns></returns>
        public override string GetExpression()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        /// A new object that is a copy of this instance.
        /// </returns>
        public override object Clone()
        {
            CompareExpression cloned = new CompareExpression(this.leftExpression, this.compareOperation, this.rightExpression);

            return cloned;
        }

        #endregion
    }
}