using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Coleccin de campos
    /// </summary>
    public class SqlFieldCollection : Collection<SqlField>, ICloneable
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public SqlFieldCollection()
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="fieldsExpression">Campos separados por ','</param>
        public SqlFieldCollection(string fieldsExpression)
        {
            AddRange(fieldsExpression);
        }

        /// <summary>
        /// Aade un nuevo campo
        /// </summary>
        /// <param name="expression">Expresin que se analizar y se convertir en un campo</param>
        /// <returns>El campo aadido</returns>
        public SqlField Add(string expression)
        {
            SqlField field = new SqlField(expression);
            base.Add(field);
            return field;
        }

        /// <summary>
        /// Aade un nuevo campo
        /// </summary>
        /// <param name="expression">Expresin que se analizar y se convertir en un campo. No debe incluir el Alias</param>
        /// <param name="alias">Alias para ese campo</param>
        /// <returns>El campo aadido</returns>		
        public SqlField Add(string expression, string alias)
        {
            SqlField field = new SqlField(expression, alias);
            base.Add(field);
            return field;
        }

        /// <summary>
        /// Aade un nuevo campo
        /// </summary>
        /// <param name="expression">Expresin que se analizar y se convertir en un campo</param>
        /// <param name="alias">Alias para ese campo</param>
        /// <param name="tableName">Tabla a la que pertenece el campo</param>
        /// <returns>El campo aadido</returns>		
        public SqlField Add(string expression, string alias, string tableName)
        {
            SqlField field = new SqlField(expression, alias, tableName);
            base.Add(field);
            return field;
        }


        /// <summary>
        /// Aade una coleccin de campos
        /// </summary>
        /// <param name="fieldExpressions">Campos separados por ','</param>
        public void AddRange(string fieldExpressions)
        {
            if (fieldExpressions == null)
                throw new ArgumentNullException("fieldExpressions");

            string[] expressionParts = fieldExpressions.Split(',');

            this.AddRange(expressionParts);
        }

        /// <summary>
        /// Adds a collection of SqlFields
        /// </summary>
        /// <param name="collection">The collection.</param>
        public void AddRange(IEnumerable<string> collection)
        {
            foreach (string expressionPart in collection)
            {
                SqlField field = new SqlField(expressionPart);
                this.Add(field);
            }
        }

        /// <summary>
        /// Adds a collection of SqlFields
        /// </summary>
        /// <param name="collection">The collection.</param>
        public void AddRange(IEnumerable<SqlField> collection)
        {
            foreach (SqlField field in collection)
            {
                this.Add(field);
            }
        }

        /// <summary>
        /// Determines whether [contains] [the specified value].
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>
        /// 	<c>true</c> if [contains] [the specified value]; otherwise, <c>false</c>.
        /// </returns>
        public bool Contains(string value)
        {
            if (value == null)
                return false;

            foreach (SqlField field in this.Items)
            {
                if (field.ToString().ToLower(System.Globalization.CultureInfo.InvariantCulture) == value.ToLower(System.Globalization.CultureInfo.InvariantCulture))
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Devuelve la representacin SQL de la coleccin de campos
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clon de la coleccin de campos
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            SqlFieldCollection cloned = new SqlFieldCollection();
            foreach (SqlField field in this.Items)
            {
                cloned.Add((SqlField) field.Clone());
            }

            return cloned;
        }

        #endregion
    }
}