using System;
using System.Collections;
using System.Collections.Generic;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Coleccin de campos y valores. 
    /// </summary>
    /// <remarks>Coleccin utilizada para insertar y actualizar campos</remarks>
    public class SqlFieldValueCollection
    {
        private Dictionary<string, FieldValue> fieldValues;

        /// <summary>
        /// Constructor
        /// </summary>
        public SqlFieldValueCollection()
        {
            fieldValues = new Dictionary<string, FieldValue>();
        }

        /// <summary>
        /// Campos
        /// </summary>
        public ICollection Fields
        {
            get { return fieldValues.Keys; }
        }

        /// <summary>
        /// Gets the field values.
        /// </summary>
        /// <value>The field values.</value>
        internal Dictionary<string, FieldValue> FieldValues
        {
            get { return fieldValues; }
        }

        /// <summary>
        /// Gets or sets the <see cref="T:Object"/> with the specified field name.
        /// </summary>
        /// <value></value>
        public object this[string fieldName]
        {
            get
            {
                if (fieldValues.ContainsKey(fieldName))
                    return fieldValues[fieldName].Value;
                else
                    throw new ArgumentOutOfRangeException("fieldName");
            }
            set
            {
                if (fieldValues.ContainsKey(fieldName))
                {
                    fieldValues[fieldName].Value = value;
                }
                else
                {
                    fieldValues.Add(fieldName, new FieldValue(value, ExpressionType.Unknown));
                }
            }
        }


        /// <summary>
        /// Adds the specified field name.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <param name="fieldValue">The value.</param>
        public void Add(string fieldName, object fieldValue)
        {
            ExpressionType expressionType = ExpressionType.Unknown;

            string stringValue = fieldValue as String;

            if (stringValue != null)
            {
                if (stringValue.StartsWith(QueryProcessor.Current.ParameterToken) || stringValue.StartsWith("[") || stringValue.Contains("+"))
                    expressionType = ExpressionType.Complex;
            }

            Add(fieldName, fieldValue, expressionType);
        }

        /// <summary>
        /// Adds the specified field name.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <param name="value">The value.</param>
        /// <param name="expressionType">Type of the expression.</param>
        public void Add(string fieldName, object value, ExpressionType expressionType)
        {
            FieldValue fieldValue = new FieldValue(value, expressionType);

            fieldValues.Add(fieldName, fieldValue);
        }

        /// <summary>
        /// Aade un campo que tomar el valor de un parmetro. 
        /// El parmetro de tendr el mismo nombre del parmetro.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        public void AddAsParameter(string fieldName)
        {
            this.Add(fieldName, QueryProcessor.Current.ParameterToken + fieldName);
        }

        /// <summary>
        /// Determines whether the specified field name contains key.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <returns>
        /// 	<c>true</c> if the specified field name contains key; otherwise, <c>false</c>.
        /// </returns>
        public bool ContainsKey(string fieldName)
        {
            return fieldValues.ContainsKey(fieldName);
        }

        /// <summary>
        /// Devuelve los campos separados por ','
        /// </summary>
        /// <returns></returns>
        public string FieldsToString()
        {
            return QueryProcessor.Current.ProcessFields(this);
        }

        /// <summary>
        /// Devuelve los valores separados por ','
        /// </summary>
        /// <param name="defaultExpressionType">Type of the default expression.</param>
        /// <returns></returns>
        public string ValuesToString(ExpressionType defaultExpressionType)
        {
            return QueryProcessor.Current.ProcessValues(this, defaultExpressionType);
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override string ToString()
        {
            return this.ToString(ExpressionType.Primitive);
        }

        /// <summary>
        /// Convert to String
        /// </summary>
        /// <param name="defaultExpressionType">Type of the default expression.</param>
        /// <returns></returns>
        public string ToString(ExpressionType defaultExpressionType)
        {
            return QueryProcessor.Current.Process(this, defaultExpressionType);
        }
    }

    /// <summary>
    /// Clase que representa el valor de un field
    /// </summary>    
    internal class FieldValue
    {
        private object currentValue;
        private ExpressionType expressionType = ExpressionType.Unknown;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:FieldValue"/> class.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="expressionType">Type of the expression.</param>
        public FieldValue(object value, ExpressionType expressionType)
        {
            this.currentValue = value;
            this.expressionType = expressionType;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:FieldValue"/> class.
        /// </summary>
        /// <param name="value">The value.</param>
        public FieldValue(object value)
        {
            this.currentValue = value;
            this.expressionType = ExpressionType.Unknown;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:FieldValue"/> class.
        /// </summary>
        public FieldValue()
        {
        }

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        /// <value>The value.</value>
        public object Value
        {
            get { return currentValue; }
            set { currentValue = value; }
        }

        /// <summary>
        /// Gets or sets the type of the expression.
        /// </summary>
        /// <value>The type of the expression.</value>
        public ExpressionType ExpressionType
        {
            get { return expressionType; }
            set { expressionType = value; }
        }

        /// <summary>
        /// Gets the string value.
        /// </summary>
        /// <returns></returns>
        public string GetStringValue()
        {
            return GetStringValue(ExpressionType.Primitive);
        }

        /// <summary>
        /// Gets the string value.
        /// </summary>
        /// <param name="defaultExpressionType">Type of the default expression.</param>
        /// <returns></returns>
        public string GetStringValue(ExpressionType defaultExpressionType)
        {
            ExpressionType effectiveExpressionType = (this.ExpressionType != ExpressionType.Unknown) ? this.ExpressionType : defaultExpressionType;

            return QueryProcessor.Current.GetExpressionStringValue(this.Value, effectiveExpressionType);
        }
    }
}