using System;
using System.Collections.Generic;
using System.Text;

namespace Avanzis.Helpers.QueryBuilder
{
    /// <summary>
    /// Clase que define una columna
    /// </summary>
    public class SQLColumn
    {
        private string name;
		private string defaultValue;
		private System.Data.SqlDbType type;
		private int length = 0;
		private bool identity;	
		private bool nullable = true;
		private bool primaryKey = false;

        /// <summary>
		/// Indica el nombre de la columna
		/// </summary>
		public string Name
		{
			get	{ return this.name; }
			set { this.name = value; }
		}
		/// <summary>
		/// Indica el tipo de base de datos de la columna
		/// </summary>
		public System.Data.SqlDbType Type
		{
			get	{ return this.type; }
			set { this.type = value; }
		}
		/// <summary>
		/// Indica la longitud mxima de los valores de la columna
		/// </summary>
		public int Length
		{
			get	{ return this.length; }
			set { this.length = value; }
		}
		/// <summary>
		/// Indica si la columna es de tipo identidad (autoincrementable)
		/// </summary>
		public bool Identity
		{
			get	{ return this.identity; }
			set { this.identity = value; }
		}
		/// <summary>
		/// Indica si los valores de la columna pueden ser nulos
		/// </summary>
		public bool Nullable
		{
			get	{ return this.nullable; }
			set { this.nullable = value; }
		}
		/// <summary>
		/// Valor por defecto de la columna
		/// </summary>
		public string DefaultValue
		{
			get { return this.defaultValue;}
			set { this.defaultValue = value; }
		}
		/// <summary>
		/// Indica si la columna es parte de la clave primaria de la tabla
		/// </summary>
		public bool PrimaryKey
		{
			get { return this.primaryKey;}
			set { this.primaryKey = value; }
		}
		
		public SQLColumn(string name, System.Data.SqlDbType type, int length) : this(name, type, length, String.Empty)
		{}

		protected SQLColumn(string name, System.Data.SqlDbType type, string defaultValue) : this(name, type, 0, defaultValue)
		{}

		protected SQLColumn(string name, System.Data.SqlDbType type, int length, string defaultValue)
		{
			this.name = name;
			this.type = type;
			this.length = length;			
			this.DefaultValue = defaultValue;
		}

		public SQLColumn(string name, System.Data.SqlDbType type, bool identity)
		{
			this.name = name;
			this.type = type;
			this.identity = identity;
		}

		public SQLColumn(string name, System.Data.SqlDbType type, bool primaryKey, bool identity)
		{
			this.name = name;
			this.type = type;
			this.PrimaryKey = primaryKey;
			this.Nullable = false;
			this.identity = identity;
		}

		public SQLColumn(string name,System.Data.SqlDbType type) : this(name,type,0)
		{
		}

		/// <summary>
		/// Obtiene la expresin SQL necesaria para crear o editar una columna
		/// </summary>
		/// <param name="tableName"></param>
		/// <returns></returns>
		public string GetSQLExpression(string tableName)
		{
			StringBuilder sb = new StringBuilder();

			sb.Append(" [" + this.Name + "] [" + this.Type.ToString() + "] ");

			if (this.Identity)
			{
				sb.Append(" IDENTITY(1,1) ");
			}
			else
			{
				if (this.Type == System.Data.SqlDbType.NVarChar)
					sb.Append(" ("+ this.Length + ") ");

				if (this.Nullable)
				{
					sb.Append(" NULL ");
				}
				else
				{
					sb.Append(" NOT NULL ");
				}
			}		
			
			return sb.ToString();
		}

		/// <summary>
		/// Obtiene el script para modificar los constraint de esa columna. Por ahora slo el valor por defecto.
		/// </summary>
		/// <param name="tableName">Nombre de la tabla</param>
		/// <returns></returns>
		public string GetConstraints(string tableName)
		{
			string constraintName = "DF_" + tableName + "_" + this.Name;
			string sql = "if OBJECT_ID ('" + constraintName + "') IS NOT NULL BEGIN";
			sql += " ALTER TABLE " + tableName + " DROP CONSTRAINT [" + constraintName + "] " + System.Environment.NewLine + " END GO " + System.Environment.NewLine;
			sql += " ALTER TABLE " + tableName + " ADD  CONSTRAINT [" + constraintName + "] DEFAULT (" + DefaultValue + ") FOR " + this.Name + System.Environment.NewLine + " GO " + System.Environment.NewLine;
			return sql;
		}

    
		


    }
}
