using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Coleccin de condiciones de bsqueda, tambin llamados predicados de bsqueda.
    /// </summary>
    public class SearchFiltersCollection : Collection<SearchPredicate>, ICloneable
    {
        private ExpressionType defaultExpressionType = ExpressionType.Primitive;

        /// <summary>
        /// Unin interna de sus filtros
        /// </summary>
        private FilterUnion defaultUnion = FilterUnion.And;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:SearchFiltersCollection"/> class.
        /// </summary>
        public SearchFiltersCollection() : base()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:SearchFiltersCollection"/> class.
        /// </summary>
        /// <param name="filterExpression">The filter expression.</param>
        public SearchFiltersCollection(string filterExpression) : this()
        {
            SearchFilter searchFilter = new SearchFilter(filterExpression);
            this.Add(searchFilter);
        }

        /// <summary>
        /// Tipo de unin de esta condicin de bsqueda con otras condiciones
        /// </summary>
        public FilterUnion DefaultUnion
        {
            get { return defaultUnion; }
            set { defaultUnion = value; }
        }

        /// <summary>
        /// Gets or sets the type of the default expression.
        /// </summary>
        /// <value>The type of the default expression.</value>
        public ExpressionType DefaultExpressionType
        {
            get { return defaultExpressionType; }
            set { defaultExpressionType = value; }
        }

        /// <summary>
        /// Aade un predicado de bsqueda con una unin determinada
        /// </summary>
        /// <param name="union"></param>
        /// <param name="searchPredicate"></param>
        /// <returns></returns>
        public SearchPredicate Add(FilterUnion union, SearchPredicate searchPredicate)
        {
            if (searchPredicate == null)
                throw new ArgumentNullException("searchPredicate");

            searchPredicate.ExternalUnion = union;
            base.Add(searchPredicate);
            return searchPredicate;
        }

        /// <summary>
        /// Aade un predicado de bsquda
        /// </summary>
        /// <param name="expression">Expresin de bsqueda</param>
        /// <returns></returns>
        public SearchPredicate Add(string expression)
        {
            SearchFilter sf = new SearchFilter(expression);
            Add(sf);
            return sf;
        }

        /// <summary>
        /// Aade un predicado de bsqueda
        /// </summary>
        /// <param name="union">Tipo de unin para este predicado</param>
        /// <param name="expression">Expresin de este predicado de bsqueda</param>
        /// <returns></returns>
        public SearchPredicate Add(FilterUnion union, string expression)
        {
            SearchFilter sf = new SearchFilter(union, false, expression);
            Add(sf);
            return sf;
        }

        /// <summary>
        /// Aade un predicado de bsqueda
        /// </summary>
        /// <param name="union">Tipo de unin para este predicado</param>
        /// <param name="negate">Si debe negar este predicado</param>
        /// <param name="expression">Expresin de este predicado de bsqueda</param>		
        /// <returns></returns>
        public SearchPredicate Add(FilterUnion union, bool negate, string expression)
        {
            SearchFilter sf = new SearchFilter(union, negate, expression);
            Add(sf);
            return sf;
        }

        /// <summary>
        /// Aade un predicado
        /// </summary>
        /// <param name="leftExpression"></param>
        /// <param name="compareOperator"></param>
        /// <param name="rightExpression"></param>
        /// <returns></returns>
        public SearchPredicate Add(object leftExpression, SqlOperator compareOperator, object rightExpression)
        {
            ExpressionType expressionType = this.defaultExpressionType;

            string stringValue = rightExpression as String;

            if (stringValue != null)
            {
                if (stringValue.StartsWith(QueryProcessor.Current.ParameterToken) || stringValue.StartsWith("[") || stringValue.StartsWith("("))
                    expressionType = ExpressionType.Complex;
            }

            return Add(leftExpression, compareOperator, rightExpression, expressionType);
        }

        /// <summary>
        /// Adds the specified left expression. It always thinks that is a primitive expression
        /// </summary>
        /// <param name="leftExpression">The left expression.</param>
        /// <param name="rightExpression">The right expression.</param>
        /// <returns></returns>
        public SearchPredicate Add(object leftExpression, object rightExpression)
        {
            if (rightExpression is SqlOperator)
                return Add(leftExpression,  (SqlOperator)rightExpression, null, ExpressionType.Unknown); ;

            return Add(leftExpression, SqlOperator.Equals, rightExpression);
        }
       
        /// <summary>
        /// Adds the specified predicate
        /// </summary>
        /// <param name="leftExpression">The left expression.</param>
        /// <param name="compareOperator">The compare operator.</param>
        /// <param name="rightExpression">The right expression.</param>
        /// <param name="rightExpressionType">Type of the right expression.</param>
        /// <returns></returns>
        public SearchPredicate Add(object leftExpression, SqlOperator compareOperator, object rightExpression, ExpressionType rightExpressionType)
        {
            SearchFilter sf = new SearchFilter(leftExpression, compareOperator, rightExpression, rightExpressionType);
            Add(sf);
            return sf;
        }

        /// <summary>
        /// Aade un predicado
        /// </summary>
        /// <param name="union">Tipo de unin para este predicado</param>		
        /// <param name="leftExpression"></param>
        /// <param name="compareOperator"></param>
        /// <param name="rightExpression"></param>
        /// <returns></returns>
        public SearchPredicate Add(FilterUnion union, object leftExpression, SqlOperator compareOperator, object rightExpression)
        {
            SearchFilter sf = new SearchFilter(union, leftExpression, compareOperator, rightExpression);
            Add(sf);
            return sf;
        }

        /// <summary>
        /// Aade un predicado
        /// </summary>
        /// <param name="union">Tipo de unin para este predicado</param>
        /// <param name="negate">Si debe negar este predicado</param>
        /// <param name="leftExpression"></param>
        /// <param name="compareOperator"></param>
        /// <param name="rightExpression"></param>
        /// <returns></returns>
        public SearchPredicate Add(FilterUnion union, bool negate, object leftExpression, SqlOperator compareOperator, object rightExpression)
        {
            SearchFilter sf = new SearchFilter(union, negate, leftExpression, compareOperator, rightExpression);
            Add(sf);
            return sf;
        }


        ///<summary>
        ///</summary>
        ///<param name="filters"></param>
        public void AddRange(IEnumerable<SearchPredicate> filters)
        {
            foreach (var filter in filters)
                Add(filter);
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Clona la coleccin de filtros de bsqueda
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            SearchFiltersCollection cloned = new SearchFiltersCollection();
            foreach (SearchPredicate condition in this.Items)
            {
                cloned.Add((SearchPredicate) condition.Clone());
            }

            return cloned;
        }

        #endregion
    }
}