using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Cada uno de los elementos de la ordenacin
    /// </summary>
    public class OrderItem : ICloneable
    {
        private OrderDirection orderDirection = OrderDirection.Asc;
        private SqlField sqlField;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="expression">Expresin de ordenacin</param>
        public OrderItem(string expression)
        {
            if (expression == null)
                throw new ArgumentNullException("expression");

            string trimmedExpression = expression.Trim();
            string workExpression = trimmedExpression.ToUpper(System.Globalization.CultureInfo.InvariantCulture);
            string fieldExpression;

            OrderDirection od = OrderDirection.Asc;
            if (workExpression.EndsWith("DESC"))
            {
                fieldExpression = trimmedExpression.Substring(0, trimmedExpression.Length - 4);
                od = OrderDirection.Desc;
            }
            else if (workExpression.EndsWith("ASC"))
            {
                fieldExpression = trimmedExpression.Substring(0, trimmedExpression.Length - 3);
                od = OrderDirection.Asc;
            }
            else
            {
                fieldExpression = trimmedExpression;
            }

            SqlField fieldFromExpression = new SqlField(fieldExpression);
            this.Field = fieldFromExpression;
            this.Direction = od;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="sqlField">Campo que se va a ordenar</param>
        /// <param name="orderDirection">Sentido de la ordenacin</param>
        public OrderItem(SqlField sqlField, OrderDirection orderDirection)
        {
            this.Field = sqlField;
            this.Direction = orderDirection;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="sqlField">Campo que se va a ordenar</param>
        /// <param name="orderDirection">Sentido de la ordenacin. "ASC" o "DESC".</param>
        public OrderItem(SqlField sqlField, string orderDirection)
        {
            if (orderDirection == null)
                throw new ArgumentNullException("orderDirection");

            orderDirection = orderDirection.Trim();
            // Convierte orderDirection en sentido de ordenacin			
            this.Field = sqlField;
            if (String.Compare(orderDirection, "ASC", true) == 0)
                this.Direction = OrderDirection.Asc;
            else if (String.Compare(orderDirection, "DESC", true) == 0)
                this.Direction = OrderDirection.Desc;
            else
                throw new ArgumentException("Valid order directions are ASC or DESC", orderDirection);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="expression">Expresin que se va a ordenar</param>
        /// <param name="orderDirection">Sentido de la ordenacin</param>
        public OrderItem(string expression, string orderDirection) : this(new SqlField(expression), orderDirection)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="expression"></param>
        /// <param name="orderDirection"></param>
        public OrderItem(string expression, OrderDirection orderDirection) : this(expression, orderDirection.ToString())
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="sqlField"></param>
        public OrderItem(SqlField sqlField) : this(sqlField, OrderDirection.Asc)
        {
        }

        /// <summary>
        /// Columna por la que se va a ordenar
        /// </summary>
        public SqlField Field
        {
            get { return sqlField; }
            set { sqlField = value; }
        }

        /// <summary>
        /// Sentido de la ordenacin. 
        /// </summary>
        public OrderDirection Direction
        {
            get { return orderDirection; }
            set { orderDirection = value; }
        }

        /// <summary>
        /// Sql de esta expresin de ordenacin
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            SqlField outputField = (SqlField) sqlField.Clone();
            outputField.Alias = null;
            string fieldExpression = outputField.ToString();
            if (!String.IsNullOrEmpty(fieldExpression.Trim()))
            {
                return outputField + " " +
                       orderDirection.ToString().ToUpper(System.Globalization.CultureInfo.InvariantCulture);
            }
            else
            {
                return "";
            }
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Clona el elemento de ordenacin
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            OrderItem cloned = new OrderItem((SqlField) this.sqlField.Clone(), this.Direction);
            return cloned;
        }

        #endregion
    }
}