using System;
using System.Collections.ObjectModel;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// OrderItems Collection
    /// </summary>
    public class OrderItemsCollection : Collection<OrderItem>, ICloneable
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public OrderItemsCollection()
        {
        }

        /// <summary>
        /// Cosntructor
        /// </summary>
        /// <param name="orderExpression">Expresiones de ordenacin, separados por ','</param>		
        public OrderItemsCollection(string orderExpression)
        {
            AddRange(orderExpression);
        }

        /// <summary>
        /// Add multiple order expressions that are separated by ','
        /// </summary>
        /// <param name="orderExpression"></param>
        public void AddRange(string orderExpression)
        {
            if (String.IsNullOrEmpty(orderExpression))
                return;

            string[] expressionParts = orderExpression.Split(',');

            foreach (string expressionPart in expressionParts)
            {
                OrderItem orderItem = new OrderItem(expressionPart);
                this.Add(orderItem);
            }
        }

        /// <summary>
        /// Adds an order expression
        /// </summary>
        /// <param name="orderExpression"></param>
        /// <returns></returns>
        public OrderItem Add(string orderExpression)
        {
            if (!String.IsNullOrEmpty(orderExpression))
            {
                OrderItem orderItem = new OrderItem(orderExpression);
                Add(orderItem);
                return orderItem;
            }
            else
            {
                return null;
            }
        }


        /// <summary>
        /// Adds an order expression from its field and its order direction string expression ("ASC" or "DESC")
        /// </summary>
        /// <param name="expression"></param>
        /// <param name="orderDirection"></param>
        /// <returns></returns>
        public OrderItem Add(string expression, string orderDirection)
        {
            OrderItem orderItem = new OrderItem(expression, orderDirection);
            Add(orderItem);
            return orderItem;
        }

        /// <summary>
        /// Adds an order expression from a field and order direction string expression ("ASC" o "DESC")
        /// </summary>
        /// <param name="sqlField"></param>
        /// <param name="orderDirection"></param>
        /// <returns></returns>
        public OrderItem Add(SqlField sqlField, string orderDirection)
        {
            OrderItem orderItem = new OrderItem(sqlField, orderDirection);
            Add(orderItem);
            return orderItem;
        }

        /// <summary>
        /// Adds an order expression from a field and an order direction
        /// </summary>
        /// <param name="sqlField"></param>
        /// <param name="orderDirection"></param>
        /// <returns></returns>
        public OrderItem Add(SqlField sqlField, OrderDirection orderDirection)
        {
            OrderItem orderItem = new OrderItem(sqlField, orderDirection);
            Add(orderItem);
            return orderItem;
        }

        /// <summary>
        /// Adds an order expression from a expression and an order direction
        /// </summary>
        /// <param name="expression"></param>
        /// <param name="orderDirection"></param>
        /// <returns></returns>
        public OrderItem Add(string expression, OrderDirection orderDirection)
        {
            OrderItem orderItem = new OrderItem(expression, orderDirection);
            Add(orderItem);
            return orderItem;
        }

        /// <summary>
        /// Adds a sql field to the order collection
        /// </summary>
        /// <param name="sqlField"></param>
        /// <returns></returns>
        public OrderItem Add(SqlField sqlField)
        {
            OrderItem orderItem = new OrderItem(sqlField);
            Add(orderItem);
            return orderItem;
        }

        /// <summary>
        /// Inserts the order expression in the specified index.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <param name="orderExpression">The order expression.</param>
        public void Insert(int index, string orderExpression)
        {
            OrderItem orderItem = new OrderItem(orderExpression);
            Insert(index, orderItem);
        }

        /// <summary>
        /// Inserts the order expression in the specified index.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <param name="expression">The expression.</param>
        /// <param name="orderDirection">The order direction.</param>
        public void Insert(int index, string expression, OrderDirection orderDirection)
        {
            OrderItem orderItem = new OrderItem(expression, orderDirection);
            Insert(index, orderItem);
        }

        /// <summary>
        /// Inserts the order expression in the specified index.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <param name="expression">The expression.</param>
        /// <param name="orderDirection">The order direction.</param>
        protected void Insert(int index, string expression, string orderDirection)
        {
            OrderItem orderItem = new OrderItem(expression, orderDirection);
            Insert(index, orderItem);
        }


        /// <summary>
        /// Inserts the order expression in the specified index.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <param name="sqlField">The SQL field.</param>
        /// <param name="orderDirection">The order direction.</param>
        public void Insert(int index, SqlField sqlField, string orderDirection)
        {
            OrderItem orderItem = new OrderItem(sqlField, orderDirection);
            Insert(index, orderItem);
        }

        /// <summary>
        /// Inserts the order expression in the specified index.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <param name="sqlField">The SQL field.</param>
        /// <param name="orderDirection">The order direction.</param>
        public void Insert(int index, SqlField sqlField, OrderDirection orderDirection)
        {
            OrderItem orderItem = new OrderItem(sqlField, orderDirection);
            Insert(index, orderItem);
        }

        /// <summary>
        /// SQL string representation
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Clones the collection
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            OrderItemsCollection cloned = new OrderItemsCollection();
            foreach (OrderItem orderItem in this.Items)
            {
                cloned.Add((OrderItem) orderItem.Clone());
            }

            return cloned;
        }

        #endregion
    }
}