using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Text;
using Avanzis.QueryFramework;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clase base de la que heredarn los procesadores 
    /// dependiendo del dialecto
    /// </summary>
    public abstract class QueryProcessor
    {
        #region Singleton

        // Singleton instance
        private static QueryProcessor instance;

        // Lock synchronization object
        private static object syncLock = new object();

        /// <summary>
        /// Gets the current QueryProcessor based on the Dialect
        /// </summary>
        /// <value>The current.</value>
        public static QueryProcessor Current
        {
            get
            {
                if (instance == null)
                {
                    lock (syncLock)
                    {
                        if (instance == null)
                        {
                            // TODO: Improve with IOC
                            switch (QueryFrameworkConfiguration.Dialect)
                            {
                                case Dialects.MSSql2005Dialect:
                                    instance = new MsSql2005Processor();
                                    break;
                                case Dialects.MSSql2000Dialect:
                                    instance = new MsSql2000Processor();
                                    break;
                                case Dialects.MySql:
                                    instance = new MySqlProcessor();
                                    break;
                                default:
                                    instance = new MsSql2005Processor();
                                    break;
                            }
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Resets the processor.
        /// </summary>
        public static void ResetProcessor()
        {
            lock (syncLock)
            {
                instance = null;
            }
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the parameter symbol.
        /// </summary>
        /// <value>The parameter symbol.</value>
        public abstract string ParameterToken { get; set; }

        /// <summary>
        /// Gets the starting quote token.
        /// </summary>
        /// <value>The starting quote token.</value>
        public abstract string StartingQuoteToken { get; }

        /// <summary>
        /// Gets the ending quote token.
        /// </summary>
        /// <value>The ending quote token.</value>
        public abstract string EndingQuoteToken { get; }

        #endregion

        #region SqlField

        /// <summary>
        /// Processes the specified field.
        /// </summary>
        /// <param name="field">The field.</param>
        /// <returns></returns>
        public string Process(SqlField field)
        {
            if (field == null)
                throw new ArgumentNullException("field");

            StringBuilder sb = new StringBuilder();
            // Nombre de la tabla
            if (field.TableName != null)
            {
                sb.Append(GetFieldValidTableName(field.TableName));
                sb.Append(".");
            }

            // Nombre del campo
            if (field.ColumnNameOrExpression != null)
            {
                sb.Append(GetFieldValidColumnNameOrExpression(field.ColumnNameOrExpression));
            }
            else
            {
                throw new ArgumentException("Column name cannot be empty", "ColumnNameOrExpression");
            }

            // Alias del campo
            if (field.Alias != null)
            {
                sb.Append(" AS ");
                sb.Append(GetFieldValidAlias(field.Alias));
            }

            return sb.ToString();
        }

        /// <summary>
        /// Obtiene un Alias vlido. Para ser vlido debe estar entre comillas simples
        /// </summary>
        /// <param name="alias"></param>
        /// <returns></returns>
        protected virtual string GetFieldValidAlias(string alias)
        {
            return GetFieldValidName(alias, "'", "'");
        }

        /// <summary>
        /// Obtiene un nombre de tabla vlido. Para ser vlido debe estar entre corchetes
        /// </summary>
        /// <param name="tableName"></param>
        /// <returns></returns>
        protected virtual string GetFieldValidTableName(string tableName)
        {
            return GetFieldValidName(tableName, StartingQuoteToken, EndingQuoteToken);
        }

        /// <summary>
        /// Obtiene una expresin vlida. 
        /// </summary>
        /// <param name="columnNameOrExpression"></param>
        /// <returns></returns>
        protected virtual string GetFieldValidColumnNameOrExpression(string columnNameOrExpression)
        {
            string resultExpression = columnNameOrExpression.Trim();

            if (resultExpression.StartsWith("("))
                return resultExpression;

            if (IsExpressionSubQueryOrFunction(resultExpression)) // Cuando empieza por SELECT
                return "(" + resultExpression + ")";
            else
                return resultExpression;
        }

        /// <summary>
        /// Obtiene un nombre vlido
        /// </summary>
        /// <param name="name">Nombre a analizar</param>
        /// <param name="startLimiter"></param>
        /// <param name="endLimiter"></param>
        /// <returns>Cadena de texto que tendr los limitadores inicial y final</returns>
        protected virtual string GetFieldValidName(string name, string startLimiter, string endLimiter)
        {
            string newName = name.Trim();
            if (newName.StartsWith(startLimiter))
                return newName;
            else if (newName.IndexOf(" ") < 0) // Si no tiene ningn espacio lo devuelve tal cual			
                return newName;
            else
                return startLimiter + newName + endLimiter;
        }

        /// <summary>
        /// Analiza una expresin y saca la que puede para tomar las propiedades del campo
        /// </summary>
        /// <param name="field">The field.</param>
        /// <param name="columnNameOrExpression">Expresin analizada</param>
        public virtual void Parse(SqlField field, string columnNameOrExpression)
        {
            if (String.IsNullOrEmpty(columnNameOrExpression))
            {
                Parse(field, columnNameOrExpression, null);
            }
            else
            {
                // Separa el Alias del resto de la expresin
                string foundAlias = null;
                string trimedColumnNameOrExpression = columnNameOrExpression;
                int lastIndexOfAs = trimedColumnNameOrExpression.ToUpper(CultureInfo.InvariantCulture).LastIndexOf(" AS ");
                // Si ha encontrado algn AS
                if (lastIndexOfAs > 0)
                {
                    // Comprueba que no est dentro de unos parntesis
                    int lastIndexOfParenthesis = trimedColumnNameOrExpression.LastIndexOf(")");
                    if (lastIndexOfAs > lastIndexOfParenthesis)
                    {
                        // Si llega aqu es el AS es vlido y tiene un Alias
                        foundAlias = trimedColumnNameOrExpression.Substring(lastIndexOfAs + 4).Trim();
                        trimedColumnNameOrExpression = trimedColumnNameOrExpression.Substring(0, lastIndexOfAs);
                    }
                }

                Parse(field, trimedColumnNameOrExpression, foundAlias);
            }
        }

        /// <summary>
        /// Analiza una expresin y saca la que puede para tomar las propiedades del campo
        /// </summary>
        /// <param name="field">The field.</param>
        /// <param name="columnNameOrExpression">Expresin analizada. No incluye el Alias</param>
        /// <param name="alias">Alias para esta expresin</param>
        public virtual void Parse(SqlField field, string columnNameOrExpression, string alias)
        {
            if (columnNameOrExpression == null)
                throw new ArgumentNullException("columnNameOrExpression");
            if (field == null)
                throw new ArgumentNullException("field");

            string foundAlias = (alias != null) ? alias.Trim() : null;
            string foundTableName = null;
            string foundColumnOrExpression;
            string trimedColumnNameOrExpression = columnNameOrExpression.Trim();

            // Si empieza por parntesis, es una expresin y no tiene una tabla asocidada
            if (trimedColumnNameOrExpression.IndexOf("(") != -1)
            {
                foundColumnOrExpression = trimedColumnNameOrExpression;
            }
            else
            {
                // Si no empieza por parntesis, debemos buscar un . para separar el nombre de la tabla del nombre del campo
                string[] expressionParts = trimedColumnNameOrExpression.Split(new char[] {'.'});
                if (expressionParts.Length == 2)
                {
                    foundTableName = expressionParts[0].Trim();
                    foundColumnOrExpression = expressionParts[1].Trim();
                }
                else
                {
                    foundTableName = null;
                    foundColumnOrExpression = trimedColumnNameOrExpression;
                }
            }

            field.TableName = foundTableName;
            field.Alias = foundAlias;
            field.ColumnNameOrExpression = foundColumnOrExpression;
        }

        #endregion

        #region SqlFieldCollection

        /// <summary>
        /// Processes the specified field collection.
        /// </summary>
        /// <param name="fieldCollection">The field collection.</param>
        /// <returns></returns>
        public virtual string Process(SqlFieldCollection fieldCollection)
        {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < fieldCollection.Count; i++)
            {
                sb.Append(fieldCollection[i].ToString());
                if (i < fieldCollection.Count - 1)
                    sb.Append(",");
            }
            return sb.ToString();
        }

        #endregion

        #region SqlFieldValueCollection

        /// <summary>
        /// Processes the specified field value collection.
        /// </summary>
        /// <param name="fieldValueCollection">The field value collection.</param>
        /// <returns></returns>
        public virtual string Process(SqlFieldValueCollection fieldValueCollection)
        {
            return Process(fieldValueCollection, ExpressionType.Primitive);
        }

        /// <summary>
        /// Processes the specified field value collection.
        /// </summary>
        /// <param name="fieldValueCollection">The field value collection.</param>
        /// <param name="defaultExpressionType">Type of the default expression.</param>
        /// <returns></returns>
        public virtual string Process(SqlFieldValueCollection fieldValueCollection, ExpressionType defaultExpressionType)
        {
            StringBuilder sb = new StringBuilder();

            int i = 0;

            foreach (string key in fieldValueCollection.FieldValues.Keys)
            {
                string fieldName = GetParsedFieldName(key);
                string fieldValue = fieldValueCollection.FieldValues[key].GetStringValue(defaultExpressionType);

                sb.Append(fieldName + " = " + fieldValue);

                if (i < fieldValueCollection.FieldValues.Keys.Count - 1)
                    sb.Append(", ");

                i++;
            }

            return sb.ToString();
        }

        /// <summary>
        /// Processes the fields.
        /// </summary>
        /// <param name="fieldValueCollection">The field value collection.</param>
        /// <returns></returns>
        public virtual string ProcessFields(SqlFieldValueCollection fieldValueCollection)
        {
            int i = 0;

            StringBuilder sb = new StringBuilder();

            foreach (string key in fieldValueCollection.FieldValues.Keys)
            {
                sb.Append(GetParsedFieldName(key));

                if (i < fieldValueCollection.FieldValues.Keys.Count - 1)
                    sb.Append(", ");

                i++;
            }
            return sb.ToString();
        }

        /// <summary>
        /// Processes the values.
        /// </summary>
        /// <param name="fieldValueCollection">The field value collection.</param>
        /// <param name="defaultExpressionType">Type of the default expression.</param>
        /// <returns></returns>
        public virtual string ProcessValues(SqlFieldValueCollection fieldValueCollection, ExpressionType defaultExpressionType)
        {
            StringBuilder sb = new StringBuilder();

            int i = 0;

            int lastIndex = fieldValueCollection.FieldValues.Values.Count - 1;

            foreach (FieldValue fieldValue in fieldValueCollection.FieldValues.Values)
            {
                sb.Append(fieldValue.GetStringValue(defaultExpressionType));

                if (i < lastIndex)
                    sb.Append(", ");
                i++;
            }

            return sb.ToString();
        }

        ///// <summary>
        ///// Gets the parsed field value.
        ///// </summary>
        ///// <param name="fieldValue">The field value.</param>
        ///// <param name="defaultExpressionType">Type of the default expression.</param>
        ///// <returns></returns>
        //private string GetParsedFieldValue(FieldValue fieldValue, ExpressionType defaultExpressionType)
        //{
        //    return fieldValue.GetStringValue(defaultExpressionType);
        //}

        /// <summary>
        /// Gets the name of the parsed field.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <returns></returns>
        private string GetParsedFieldName(string fieldName)
        {
            if (fieldName.IndexOf(".") < 0 && !fieldName.StartsWith(StartingQuoteToken))
            {
                fieldName = StartingQuoteToken + fieldName + EndingQuoteToken;
            }
            return fieldName;
        }

        #endregion

        #region Operators

        /// <summary>
        /// Processes the specified SQL operator.
        /// </summary>
        /// <param name="sqlOperator">The SQL operator.</param>
        /// <returns></returns>
        public virtual string Process(SqlOperator sqlOperator)
        {
            string operatorString;

            switch (sqlOperator)
            {
                case SqlOperator.Contains:
                    operatorString = "CONTAINS";
                    break;
                case SqlOperator.Distinct:
                    operatorString = "!=";
                    break;
                case SqlOperator.Exists:
                    operatorString = "EXISTS";
                    break;
                case SqlOperator.Greater:
                    operatorString = ">";
                    break;
                case SqlOperator.GreaterOrEqual:
                    operatorString = ">=";
                    break;
                case SqlOperator.In:
                    operatorString = "IN";
                    break;
                case SqlOperator.NotIn:
                    operatorString = "NOT IN";
                    break;
                case SqlOperator.IsNotNull:
                    operatorString = "IS NOT NULL";
                    break;
                case SqlOperator.IsNull:
                    operatorString = "IS NULL";
                    break;
                case SqlOperator.Less:
                    operatorString = "<";
                    break;
                case SqlOperator.LessOrEqual:
                    operatorString = "<=";
                    break;
                case SqlOperator.Like:
                    operatorString = "LIKE";
                    break;
                case SqlOperator.NotLike:
                    operatorString = "NOT LIKE";
                    break;
                case SqlOperator.Is:
                    operatorString = "IS";
                    break;
                case SqlOperator.Equals:
                    operatorString = "=";
                    break;
                default:
                    throw new NotImplementedException();
            }
            return operatorString;
        }

        #endregion

        #region Subquery & Functions

        /// <summary>
        /// Si es una subquery, debe devolverla entre parntesis
        /// </summary>
        /// <param name="expression"></param>
        /// <returns></returns>
        protected virtual bool IsExpressionSubQueryOrFunction(string expression)
        {
            if (String.IsNullOrEmpty(expression))
                return false;

            expression = expression.ToUpper(CultureInfo.InvariantCulture);
            if (expression.StartsWith("SELECT "))
                return true;
            else if (expression.StartsWith("CASE "))
                return true;
            else
                return false;
        }

        #endregion

        #region CompareExpression

        /// <summary>
        /// Processes the specified expression.
        /// </summary>
        /// <param name="expression">The expression.</param>
        /// <returns></returns>
        public virtual string Process(CompareExpression expression)
        {
            if (expression == null)
                throw new ArgumentNullException("expression");

            string startLimiter = "";
            string endLimiter = "";

            StringBuilder sb = new StringBuilder();

            bool hasUniqueExpression = String.IsNullOrEmpty(expression.LeftExpression);
            if (!hasUniqueExpression)
            {
                sb.Append(expression.LeftExpression);
                sb.Append(" ");
            }

            expression.CompareOperation = GetValidatedCompareOperator(expression.CompareOperation, expression.RightExpression);

            // Aade el operador
            sb.Append(Process(expression.CompareOperation));

            if (!IsUnaryOperator(expression.CompareOperation))
            {
                sb.Append(" ");

                GetSqlOperatorLimiters(expression.CompareOperation, ref startLimiter, ref endLimiter);

                if (!expression.RightExpression.StartsWith(startLimiter))
                    sb.Append(startLimiter);

                sb.Append(expression.RightExpression);

                if (!expression.RightExpression.EndsWith(endLimiter))
                    sb.Append(endLimiter);
            }

            return sb.ToString();
        }

        /// <summary>
        /// Comprueba que el operador es vlido para la expresin
        /// adems lo puede modificar en funcin de la expresin (Equals -> Is) 
        /// </summary>
        /// <param name="compareOperator">The compare operator.</param>
        /// <param name="rightExpression">The right expression.</param>
        protected virtual SqlOperator GetValidatedCompareOperator(SqlOperator compareOperator, string rightExpression)
        {
            // Gestiona el caso especial para cuando el la expresin de la derecha es null
            if (rightExpression == null || rightExpression == QueryExpression.NullString)
            {
                if (compareOperator == SqlOperator.Equals)
                    compareOperator = SqlOperator.Is;
            }
            else
            {
                switch (compareOperator)
                {
                    case SqlOperator.Is:
                        if (rightExpression != QueryExpression.NullString) //  || rightExpression == null)
                            throw new InvalidOperationException("Only use IS opertar with null values");
                        break;
                    default:
                        break;
                }
            }

            return compareOperator;
        }

        #endregion

        #region SearchFiltersCollection

        /// <summary>
        /// Processes the specified search filters collection.
        /// </summary>
        /// <param name="searchFiltersCollection">The search filters collection.</param>
        /// <returns></returns>
        public virtual string Process(SearchFiltersCollection searchFiltersCollection)
        {
            if (searchFiltersCollection == null)
                return "";

            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < searchFiltersCollection.Count; i++)
            {
                SearchPredicate predicate = searchFiltersCollection[i];
                string predicateExpression = predicate.ToString();
                if (!String.IsNullOrEmpty(predicateExpression))
                {
                    if (i > 0)
                    {
                        sb.Append(" ");
                        if (predicate.ExternalUnion == FilterUnion.NotSet)
                        {
                            if (searchFiltersCollection.DefaultUnion != FilterUnion.NotSet)
                                sb.Append(searchFiltersCollection.DefaultUnion.ToString().ToUpperInvariant());
                            else
                                sb.Append(FilterUnion.And.ToString().ToUpperInvariant());
                        }
                        else
                        {
                            sb.Append(predicate.ExternalUnion.ToString().ToUpperInvariant());
                        }
                        sb.Append(" ");
                    }
                    if (predicate.Negative)
                        sb.Append("NOT ");
                    sb.Append("(");
                    sb.Append(predicateExpression);
                    sb.Append(")");
                }
            }
            return sb.ToString();
        }

        #endregion

        #region SearchCondition

        /// <summary>
        /// Processes the specified search condition.
        /// </summary>
        /// <param name="searchCondition">The search condition.</param>
        /// <returns></returns>
        public virtual string Process(SearchCondition searchCondition)
        {
            if (searchCondition == null)
                throw new ArgumentNullException("searchCondition");

            StringBuilder sb = new StringBuilder();
            searchCondition.Filters.DefaultUnion = searchCondition.DefaultFiltersUnion;
            if (searchCondition.Negative)
                sb.Append("NOT ");
            sb.Append(searchCondition.Filters.ToString());
            return sb.ToString();
        }

        #endregion

        #region Expression

        /// <summary>
        /// Gets the string value.
        /// </summary>
        /// <param name="expression">The expression.</param>
        /// <returns></returns>
        public virtual string GetExpressionStringValue(object expression)
        {
            return GetExpressionStringValue(expression, ExpressionType.Unknown);
        }

        /// <summary>
        /// Gets the string value.
        /// </summary>
        /// <param name="expression">The expression.</param>
        /// <param name="escapeString">if set to <c>true</c> [escape string].</param>
        /// <returns></returns>
        public virtual string GetExpressionStringValue(object expression, bool escapeString)
        {
            ExpressionType expressionType = (escapeString) ? ExpressionType.Primitive : ExpressionType.Complex;
            return GetExpressionStringValue(expression, expressionType);
        }

        /// <summary>
        /// Gets the string value.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="expressionType">Type of the expression.</param>
        /// <returns></returns>
        public virtual string GetExpressionStringValue(object value, ExpressionType expressionType)
        {
            string stringValue;
            // NumberFormatInfo nfi = new CultureInfo( "en-US", false ).NumberFormat;
            NumberFormatInfo nfi = new NumberFormatInfo();
            nfi.NumberDecimalSeparator = ".";

            if (value is bool)
            {
                stringValue = ((bool) value) ? "1" : "0";
            }
            else if (value is DateTime)
            {
                
                stringValue = GetDateStringValue((DateTime)value);
            }
            else if (value == null)
            {
                stringValue = QueryExpression.NullString;
            }
            else if (value is Guid)
            {
                stringValue = "'" + value + "'";
            }
            else if (value is String) // Aqu es donde podemos tener problemas
            {
                stringValue = value.ToString().Trim();

                if (expressionType == ExpressionType.Unknown)
                {
                    expressionType = Current.GetBestMatchedExpressionType(stringValue);
                }

                // Si es de tipo primitivo debe hacer el parse
                if (expressionType == ExpressionType.Primitive)
                {
                    if (!stringValue.StartsWith("'"))
                    {
                        if (!Current.IsSqlFunction(stringValue))
                        {
                            stringValue = stringValue.Replace("'", "''");
                            stringValue = "'" + stringValue + "'";
                        }
                    }
                }
            }
            else if (value is Double)
            {
                stringValue = ((double) value).ToString("G", nfi);
            }
            else if (value is float)
            {
                stringValue = ((float) value).ToString("G", nfi);
            }
            else if (value is Builder)
            {
                stringValue = "(" + value + ")";
            }
            else if (value is IEnumerable)
            {
                IEnumerable collection = value as IEnumerable;
                List<string> values = new List<string>();
                foreach (var item in collection)
                {
                    values.Add(GetExpressionStringValue(item, ExpressionType.Primitive));
                }
                stringValue = String.Join(",", values.ToArray());
            }
            else
            {
                stringValue = value.ToString();
            }
            return stringValue;
        }

        protected virtual string GetDateStringValue(DateTime value)
        {
            return "'" + value.ToString("yyyyMMdd HH:mm:ss") + "'";
        }

        #endregion

        #region Joins

        /// <summary>
        /// Processes the specified join table.
        /// </summary>
        /// <param name="joinTable">The join table.</param>
        /// <returns></returns>
        public virtual string Process(JoinTable joinTable)
        {
            if (joinTable == null)
                throw new ArgumentNullException("joinTable");

            StringBuilder sb = new StringBuilder();
            // Join
            sb.Append(GetJoinTypeString(joinTable.JoinType));
            sb.Append(" ");
            // TableSource
            if (joinTable.TableSource != null)
            {
                sb.Append(joinTable.TableSource.ToString());
            }
            else
            {
                throw new ArgumentException("Join table source not set", "TableSource");
            }
            // Condicin de bsqueda
            if (joinTable.SearchCondition != null)
            {
                sb.Append(" ON (");
                sb.Append(joinTable.SearchCondition.ToString());
                sb.Append(")");
            }

            // Alias

            return sb.ToString();
        }

        #endregion

        #region From

        /// <summary>
        /// Processes the specified from clause.
        /// </summary>
        /// <param name="fromClause">From clause.</param>
        /// <returns></returns>
        public virtual string Process(FromClause fromClause)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("FROM ");
            sb.Append(fromClause.MainTableSource.ToString());
            for (int i = 0; i < fromClause.Joins.Count; i++)
            {
                TableSource tableSource = fromClause.Joins[i];
                sb.Append(" ");
                sb.Append(tableSource.ToString());
            }
            return sb.ToString();
        }

        #endregion

        #region Select Clause

        /// <summary>
        /// Processes the specified select clause.
        /// </summary>
        /// <param name="selectClause">The select clause.</param>
        /// <returns></returns>
        public abstract string Process(SelectClause selectClause);

        #endregion

        #region Select Builder

        /// <summary>
        /// Processes the specified select builder.
        /// </summary>
        /// <param name="selectBuilder">The select builder.</param>
        /// <returns></returns>
        public abstract string Process(SelectBuilder selectBuilder);

        #endregion

        #region Where Clause

        /// <summary>
        /// Processes the specified where clause.
        /// </summary>
        /// <param name="whereClause">The where clause.</param>
        /// <returns></returns>
        public virtual string Process(WhereClause whereClause)
        {
            StringBuilder sb = new StringBuilder();
            string searchConditionResult = whereClause.SearchCondition.ToString();
            if (searchConditionResult.Length > 0)
            {
                sb.Append("WHERE ");
                sb.Append(searchConditionResult);
            }
            return sb.ToString();
        }

        #endregion

        #region Delete Clause

        /// <summary>
        /// Processes the specified delete clause.
        /// </summary>
        /// <param name="deleteClause">The delete clause.</param>
        /// <returns></returns>
        public virtual string Process(DeleteClause deleteClause)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("DELETE ");
            return sb.ToString();
        }

        #endregion

        #region TableSource

        /// <summary>
        /// Processes the specified table source.
        /// </summary>
        /// <param name="tableSource">The table source.</param>
        /// <returns></returns>
        public virtual string Process(TableSource tableSource)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append(GetParsedTableName(tableSource.TableName));

            if (tableSource.UseAlias && !String.IsNullOrEmpty(tableSource.Alias))
            {
                sb.Append(" AS ");
                sb.Append(tableSource.Alias);
            }
            return sb.ToString();
        }

        /// <summary>
        /// Obtiene el nombre de la tabla entre corchetes
        /// </summary>
        /// <param name="currentTableName"></param>
        /// <returns></returns>
        protected string GetParsedTableName(string currentTableName)
        {
            if (String.IsNullOrEmpty(currentTableName))
                return currentTableName;

            currentTableName = currentTableName.Trim();

            if (!currentTableName.StartsWith("(") && currentTableName.IndexOf(".") == -1)
            {
                if (!currentTableName.StartsWith(StartingQuoteToken))
                {
                    currentTableName = StartingQuoteToken + currentTableName;
                }
                if (!currentTableName.EndsWith(EndingQuoteToken))
                {
                    currentTableName += EndingQuoteToken;
                }
            }

            return currentTableName;
        }

        /// <summary>
        /// Parses the specified expression and updates the table source
        /// </summary>
        /// <param name="tableSource">The table source.</param>
        /// <param name="tableName">Name of the table.</param>
        public virtual void Parse(TableSource tableSource, string tableName)
        {
            tableName = tableName.Trim();

            int asIndex = tableName.ToLowerInvariant().LastIndexOf(" as ");

            if (asIndex > 0)
            {
                tableSource.TableName = tableName.Substring(0, asIndex).Trim();
                tableSource.Alias = tableName.Substring(asIndex + 4).Trim();
            }
            else
            {
                tableSource.TableName = tableName;
                tableSource.Alias = null;
            }
        }

        #endregion

        #region Search Predicate

        /// <summary>
        /// Processes the specified search predicate.
        /// </summary>
        /// <param name="searchPredicate">The search predicate.</param>
        /// <returns></returns>
        public virtual string Process(SearchPredicate searchPredicate)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(searchPredicate.GetPredicateExpression());
            return sb.ToString();
        }

        #endregion

        #region OrderItemsCollection

        /// <summary>
        /// Processes the specified order items collection.
        /// </summary>
        /// <param name="orderItemsCollection">The order items collection.</param>
        /// <returns></returns>
        public virtual string Process(OrderItemsCollection orderItemsCollection)
        {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < orderItemsCollection.Count; i++)
            {
                sb.Append(orderItemsCollection[i].ToString());
                if (i < orderItemsCollection.Count - 1)
                    sb.Append(",");
            }
            return sb.ToString();
        }

        #endregion

        #region GroupBy Clause

        /// <summary>
        /// Processes the specified group by clause.
        /// </summary>
        /// <param name="groupByClause">The group by clause.</param>
        /// <returns></returns>
        public virtual string Process(GroupByClause groupByClause)
        {
            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < groupByClause.Fields.Count; i++)
            {
                // Si es el primero, aade el Group By
                if (i == 0)
                    sb.Append("GROUP BY ");
                // Aade la expresin quitndole el Alias
                SqlField field = ((SqlField) groupByClause.Fields[i].Clone());
                field.Alias = null;
                sb.Append(field.ToString());
                // Si no es el ltimo, aade la coma
                if (i < groupByClause.Fields.Count - 1)
                    sb.Append(", ");
            }

            return sb.ToString();
        }

        #endregion

        #region OrderBy Clause

        /// <summary>
        /// Processes the specified order by clause.
        /// </summary>
        /// <param name="orderByClause">The order by clause.</param>
        /// <returns></returns>
        public virtual string Process(OrderByClause orderByClause)
        {
            StringBuilder sb = new StringBuilder();
            if (orderByClause.Fields.Count > 0)
            {
                for (int i = 0; i < orderByClause.Fields.Count; i++)
                {
                    OrderItem orderItem = orderByClause.Fields[i];
                    sb.Append(orderItem.ToString());
                    // Si no es el ltimo aade la coma
                    if (i < orderByClause.Fields.Count - 1)
                        sb.Append(",");
                }

                sb.Insert(0, "ORDER BY ");
            }
            return sb.ToString();
        }

        #endregion

        #region Having

        /// <summary>
        /// Processes the specified clause.
        /// </summary>
        /// <param name="clause">The clause.</param>
        /// <returns></returns>
        public virtual string Process(HavingClause clause)
        {
            StringBuilder sb = new StringBuilder();
            string searchConditionResult = clause.SearchCondition.ToString();
            if (searchConditionResult.Length > 0)
            {
                sb.Append("HAVING ");
                sb.Append(searchConditionResult);
            }
            return sb.ToString();
        }

        #endregion

        #region ConnectionStringBuilder

        /// <summary>
        /// Processes the specified connection string builder.
        /// </summary>
        /// <param name="connectionStringBuilder">The connection string builder.</param>
        /// <returns></returns>
        public virtual string Process(ConnectionStringBuilder connectionStringBuilder)
        {
            StringBuilder sb = new StringBuilder();

            foreach (string key in connectionStringBuilder.ConnectionStringCollection.Keys)
            {
                sb.Append(key);
                sb.Append("=");
                sb.Append(connectionStringBuilder.ConnectionStringCollection[key]);
                sb.Append(";");
            }
            sb.Remove(sb.Length - 1, 1);

            return sb.ToString();
        }

        #endregion

        #region DeleteBuilder

        /// <summary>
        /// Processes the specified delete builder.
        /// </summary>
        /// <param name="deleteBuilder">The delete builder.</param>
        /// <returns></returns>
        public virtual string Process(DeleteBuilder deleteBuilder)
        {
            if (string.IsNullOrEmpty(deleteBuilder.TableName))
                throw new ArgumentException("Table name cannot be empty", "TableName");

            StringBuilder sb = new StringBuilder();
            AppendQueryExpression("DELETE", deleteBuilder.DeleteClause.ToString(), sb, true);
            AppendQueryExpression("FROM", deleteBuilder.From.ToString(), sb, true);
            AppendQueryExpression("WHERE", deleteBuilder.Where.ToString(), sb, !deleteBuilder.AllowDeleteAll);
            return sb.ToString();
        }

        #endregion

        #region InsertBuilder

        /// <summary>
        /// Processes the specified insert builder.
        /// </summary>
        /// <param name="insertBuilder">The insert builder.</param>
        /// <returns></returns>
        public virtual string Process(InsertBuilder insertBuilder)
        {
            return insertBuilder.Insert.ToString();
        }

        #endregion

        #region UpdateClause

        /// <summary>
        /// Processses the specified update clause.
        /// </summary>
        /// <param name="updateClause">The update clause.</param>
        /// <returns></returns>
        public virtual string Processs(UpdateClause updateClause)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append("UPDATE ");
            sb.Append(updateClause.TableSource);
            sb.Append(" SET ");
            sb.Append(updateClause.FieldValues.ToString());

            return sb.ToString();
        }

        #endregion

        #region InsertClause

        /// <summary>
        /// Processes the specified insert clause.
        /// </summary>
        /// <param name="insertClause">The insert clause.</param>
        /// <param name="expressionType">Type of the expression.</param>
        /// <returns></returns>
        public virtual string Process(InsertClause insertClause, ExpressionType expressionType)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append("INSERT INTO ");
            sb.Append(insertClause.TableSource.ToString());
            sb.Append(" (");
            sb.Append(insertClause.FieldValues.FieldsToString());
            sb.Append(") VALUES (");
            sb.Append(insertClause.FieldValues.ValuesToString(expressionType));
            sb.Append(")");

            return sb.ToString();
        }

        #endregion

        #region Update Builder

        /// <summary>
        /// Processes the specified update builder.
        /// </summary>
        /// <param name="updateBuilder">The update builder.</param>
        /// <returns></returns>
        public virtual string Process(UpdateBuilder updateBuilder)
        {
            // Comprueba que todo est bien			
            StringBuilder sb = new StringBuilder();
            AppendQueryExpression("Update", updateBuilder.Update.ToString(), sb, true);
            AppendQueryExpression(" Where", updateBuilder.Where.ToString(), sb, false);
            return sb.ToString();
        }

        #endregion

        #region UpdateClause

        /// <summary>
        /// Processes the specified update clause.
        /// </summary>
        /// <param name="updateClause">The update clause.</param>
        /// <returns></returns>
        public virtual string Process(UpdateClause updateClause)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append("UPDATE ");
            sb.Append(updateClause.TableSource);
            sb.Append(" SET ");
            sb.Append(updateClause.FieldValues.ToString());

            return sb.ToString();
        }

        #endregion

        #region Builder Functions

        /// <summary>
        /// Aade una expresin
        /// </summary>
        /// <param name="expressionName"></param>
        /// <param name="expression"></param>
        /// <param name="sb"></param>
        /// <param name="throwExceptionIfEmpty"></param>
        protected virtual void AppendQueryExpression(string expressionName, string expression, StringBuilder sb, bool throwExceptionIfEmpty)
        {
            if (sb == null)
                throw new ArgumentNullException("sb");

            if (!string.IsNullOrEmpty(expression))
            {
                sb.Append(expression);
                sb.Append("\n");
            }
            else
            {
                if (throwExceptionIfEmpty)
                    throw new ArgumentNullException(expressionName, String.Format(CultureInfo.InvariantCulture, "Clause must be defined '{0}'", expressionName));
            }
        }

        #endregion

        /// <summary>
        /// Funciones definidas en este dialecto
        /// </summary>
        private List<string> sqlFunctions;

        /// <summary>
        /// Gets the SQL functions.
        /// </summary>
        /// <value>The SQL functions.</value>
        protected virtual List<string> SqlFunctions
        {
            get
            {
                if (sqlFunctions == null)
                {
                    sqlFunctions = new List<string>();
                    // Aggregate Functions
                    sqlFunctions.Add("AVG");
                    sqlFunctions.Add("COUNT");
                    sqlFunctions.Add("COUNT");
                    sqlFunctions.Add("MIN");
                    sqlFunctions.Add("MAX");
                    sqlFunctions.Add("SUM");
                    sqlFunctions.Add("CHECKSUM");
                    sqlFunctions.Add("CHECKSUM_AGG");
                    sqlFunctions.Add("STDEV");
                    sqlFunctions.Add("STDEVP");
                    sqlFunctions.Add("COUNT_BIG");
                    sqlFunctions.Add("GROUPING");
                    sqlFunctions.Add("VAR");
                    sqlFunctions.Add("VARP");
                    // Date and Time Functions
                    sqlFunctions.Add("DATEADD");
                    sqlFunctions.Add("DATEDIFF");
                    sqlFunctions.Add("DATENAME");
                    sqlFunctions.Add("DATEPART");
                    sqlFunctions.Add("DAY");
                    sqlFunctions.Add("GETDATE");
                    sqlFunctions.Add("GETUTCDATE");
                    sqlFunctions.Add("MONTH");
                    sqlFunctions.Add("YEAR");
                    // Math Functions
                    sqlFunctions.Add("ABS");
                    sqlFunctions.Add("ACOS");
                    sqlFunctions.Add("ASIN");
                    sqlFunctions.Add("ATAN");
                    sqlFunctions.Add("ATN2");
                    sqlFunctions.Add("CEILING");
                    sqlFunctions.Add("COS");
                    sqlFunctions.Add("COT");
                    sqlFunctions.Add("DEGREES");
                    sqlFunctions.Add("EXP");
                    sqlFunctions.Add("FLOOR");
                    sqlFunctions.Add("LOG");
                    sqlFunctions.Add("LOG10");
                    sqlFunctions.Add("PI");
                    sqlFunctions.Add("POWER");
                    sqlFunctions.Add("RADIANS");
                    sqlFunctions.Add("RAND");
                    sqlFunctions.Add("ROUND");
                    sqlFunctions.Add("SIGN");
                    sqlFunctions.Add("SIN");
                    sqlFunctions.Add("SQRT");
                    sqlFunctions.Add("SQUARE");
                    sqlFunctions.Add("TAN");
                    // String Functions
                    sqlFunctions.Add("ASCII");
                    sqlFunctions.Add("CHAR");
                    sqlFunctions.Add("CHARINDEX");
                    sqlFunctions.Add("DIFFERENCE");
                    sqlFunctions.Add("LEFT");
                    sqlFunctions.Add("LEN");
                    sqlFunctions.Add("LOWER");
                    sqlFunctions.Add("LTRIM");
                    sqlFunctions.Add("NCHAR");
                    sqlFunctions.Add("PATINDEX");
                    sqlFunctions.Add("QUOTENAME");
                    sqlFunctions.Add("REPLACE");
                    sqlFunctions.Add("REPLICATE");
                    sqlFunctions.Add("REVERSE");
                    sqlFunctions.Add("RIGHT");
                    sqlFunctions.Add("RTRIM");
                    sqlFunctions.Add("SOUNDEX");
                    sqlFunctions.Add("SPACE");
                    sqlFunctions.Add("STR");
                    sqlFunctions.Add("STUFF");
                    sqlFunctions.Add("SUBSTRING");
                    sqlFunctions.Add("UNICODE");
                    sqlFunctions.Add("UPPER");
                }

                return sqlFunctions;
            }
        }

        /// <summary>
        /// Determines whether [is SQL function] [the specified expression].
        /// </summary>
        /// <param name="expression">The expression.</param>
        /// <returns>
        /// 	<c>true</c> if [is SQL function] [the specified expression]; otherwise, <c>false</c>.
        /// </returns>
        protected virtual bool IsSqlFunction(string expression)
        {
            if (!string.IsNullOrEmpty(expression))
            {
                expression = expression.Trim().ToUpper(CultureInfo.InvariantCulture);

                if (expression.IndexOf("(") > 0)
                {
                    if (SqlFunctions != null && SqlFunctions.Count > 0)
                    {
                        foreach (string sqlFunction in SqlFunctions)
                        {
                            if (expression.StartsWith(sqlFunction))
                            {
                                return true;
                            }
                        }
                    }
                }
            }
            return false;
        }

        /// <summary> 
        /// Traduce la enumeracin JoinType a SQL
        /// </summary>
        /// <param name="joinType"></param>
        /// <returns></returns>
        protected virtual string GetJoinTypeString(JoinTypes joinType)
        {
            switch (joinType)
            {
                case JoinTypes.Full:
                    return "FULL JOIN";
                case JoinTypes.Left:
                    return "LEFT JOIN";
                case JoinTypes.Right:
                    return "RIGHT JOIN";
                case JoinTypes.Inner:
                default:
                    return "INNER JOIN";
            }
        }


        /// <summary>
        /// Gets the type of the operator expression.
        /// </summary>
        /// <param name="sqlOperator">The SQL operator.</param>
        /// <returns></returns>
        public virtual ExpressionType GetOperatorExpressionType(SqlOperator sqlOperator)
        {
            ExpressionType expressionType = ExpressionType.Primitive;
            switch (sqlOperator)
            {
                case SqlOperator.In:
                case SqlOperator.Exists:
                case SqlOperator.Contains:
                    expressionType = ExpressionType.Complex;
                    break;
            }
            return expressionType;
        }

        ///<summary> 
        ///</summary>
        ///<param name="sqlOperator"></param>
        ///<returns>True if operator is unary</returns>
        public bool IsUnaryOperator(SqlOperator sqlOperator)
        {
            switch (sqlOperator)
            {
                case SqlOperator.IsNull:
                case SqlOperator.IsNotNull:
                    return true;
            }
            return false;
        }


        /// <summary>
        /// Gets the ExpressionType that best matches this expression
        /// </summary>
        /// <param name="expression">The expression.</param>
        /// <returns></returns>
        public virtual ExpressionType GetBestMatchedExpressionType(string expression)
        {
            if (expression == null)
                throw new ArgumentNullException("expression");

            ExpressionType expressionType = ExpressionType.Unknown;

            if (expression.StartsWith("%") && expression.EndsWith("%"))
                expressionType = ExpressionType.Primitive;
            else if (expression.IndexOf(' ') >= 0) // Si tiene espacios, 'casi' seguro que es complejo                
                expressionType = ExpressionType.Complex;
            else if (expression.StartsWith(ParameterToken))
                expressionType = ExpressionType.Complex;
            else if (expression.StartsWith(StartingQuoteToken) || expression.IndexOf("(") >= 0)
                expressionType = ExpressionType.Complex;
            else if (expression.StartsWith("'") && expression.EndsWith("'"))
                expressionType = ExpressionType.Primitive;
            else if (IsSqlFunction(expression))
                expressionType = ExpressionType.Complex;

            return expressionType;
        }

        /// <summary>
        /// Gets the fixed parameter name for this Database
        /// </summary>
        /// <param name="parameterName">Name of the parameter.</param>
        /// <returns></returns>
        public string GetParameterName(string parameterName)
        {
            if (parameterName == null) throw new ArgumentNullException("parameterName");

            parameterName = parameterName.Trim();

            if (!parameterName.StartsWith(Current.ParameterToken))
            {
                parameterName = Current.ParameterToken + parameterName;
            }
            return parameterName;
        }

        /// <summary>
        /// Aade los limitadores necesarios para un cierto operacin
        /// </summary>
        /// <param name="sqlOperator"></param>
        /// <param name="startLimiter"></param>
        /// <param name="endLimiter"></param>
        protected virtual void GetSqlOperatorLimiters(SqlOperator sqlOperator, ref string startLimiter, ref string endLimiter)
        {
            switch (sqlOperator)
            {
                case SqlOperator.Contains:
                case SqlOperator.Exists:
                case SqlOperator.In:
                case SqlOperator.NotIn:
                case SqlOperator.IsNull:
                case SqlOperator.IsNotNull:
                    startLimiter = "(";
                    endLimiter = ")";
                    break;

                default:
                    startLimiter = "";
                    endLimiter = "";
                    break;
            }
        }
    }
}