using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clases para gener consultas SQL
    /// </summary>
    public class SelectBuilder : Builder, ICloneable
    {
        private FromClause from;
        private GroupByClause groupBy;
        private HavingClause having;
        private OrderByClause orderBy;
        private SelectClause select;
        private WhereClause where;

        /// <summary>
        /// Constructor
        /// </summary>
        public SelectBuilder()
        {
        }


        /// <summary>
        /// Initializes a new instance of the <see cref="SelectBuilder"/> class.
        /// </summary>
        /// <param name="fromExpression">From expression.</param>
        public SelectBuilder(string fromExpression)
        {
            this.From = new FromClause(fromExpression);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="selectFields">Campos que se devolvern</param>
        /// <param name="fromExpression">Expresin FROM</param>
        public SelectBuilder(string selectFields, string fromExpression)
        {
            this.select = new SelectClause(selectFields);
            this.from = new FromClause(fromExpression);
        }


        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="selectFields">Campos que se devolvern</param>
        /// <param name="fromExpression">From expression.</param>
        /// <param name="whereCondition">The where condition.</param>
        public SelectBuilder(string selectFields, string fromExpression, string whereCondition)
        {
            this.select = new SelectClause(selectFields);
            this.from = new FromClause(fromExpression);
            this.Where.Filters.Add(whereCondition);
        }

        /// <summary>
        /// Clasula SELECT
        /// </summary>
        public SelectClause Select
        {
            get { return select ?? (select = new SelectClause()); }
            set { select = value; }
        }

        /// <summary>
        /// Clasula FROM
        /// </summary>
        public FromClause From
        {
            get { return from ?? (from = new FromClause()); }
            set { from = value; }
        }

        /// <summary>
        /// Clasula WHERE
        /// </summary>
        public WhereClause Where
        {
            get
            {
                if (where == null)
                {
                    where = new WhereClause();
                    where.Filters.DefaultExpressionType = ExpressionType.Unknown;
                }
                return where;
            }
            set { where = value; }
        }

        /// <summary>
        /// Clasula GROUPBY
        /// </summary>
        public GroupByClause GroupBy
        {
            get { return groupBy ?? (groupBy = new GroupByClause()); }
            set { groupBy = value; }
        }

        /// <summary>
        /// Clasula HAVING
        /// </summary>
        public HavingClause Having
        {
            get
            {
                if (having == null)
                    having = new HavingClause();
                return having;
            }
            set { having = value; }
        }

        /// <summary>
        /// Clasula OrderBy
        /// </summary>
        public OrderByClause OrderBy
        {
            get
            {
                if (orderBy == null)
                    orderBy = new OrderByClause();
                return orderBy;
            }
            set { orderBy = value; }
        }

        /// <summary>
        /// Campos que se seleccionarn
        /// </summary>
        public SqlFieldCollection Fields
        {
            get { return this.Select.Fields; }
        }

        /// <summary>
        /// Nombre de la tabla donde obtiene los datos
        /// </summary>
        [Obsolete("Use TableName as instead of FromTableName")]
        public string FromTableName
        {
            get { return TableName; }
            set { TableName = value; }
        }

        /// <summary>
        /// Nombre de la tabla a actualizar
        /// </summary>
        public override string TableName
        {
            get { return this.From.TableName; }
            set { this.From.TableName = value; }
        }

        /// <summary>
        /// Alias de la tabla seleccionada
        /// </summary>
        public string FromTableAlias
        {
            get { return this.From.Alias; }
            set { this.From.Alias = value; }
        }

        /// <summary>
        /// Joins esta consulta
        /// </summary>
        public JoinTableCollection Joins
        {
            get { return this.From.Joins; }
        }

        /// <summary>
        /// Filtros
        /// </summary>
        public SearchFiltersCollection Filters
        {
            get { return this.Where.Filters; }
        }

        /// <summary>
        /// Unin por defecto los filtros del WHERE
        /// </summary>
        public FilterUnion Union
        {
            get { return this.Where.Union; }
            set { this.Where.Union = value; }
        }

        /// <summary>
        /// Campos por los que agrupar
        /// </summary>
        public SqlFieldCollection GroupByFields
        {
            get { return this.GroupBy.Fields; }
        }

        /// <summary>
        /// Campos por los que ordenar
        /// </summary>
        public OrderItemsCollection OrderFields
        {
            get { return this.OrderBy.Fields; }
        }

        /// <summary>
        /// Gets or sets the limit.
        /// </summary>
        /// <value>The limit.</value>
        public int Limit
        {
            get { return this.Select.Limit; }
            set { this.Select.Limit = value; }
        }

        /// <summary>
        /// Gets or sets the Offset.
        /// </summary>
        /// <value>The limit.</value>
        public int Offset
        {
            get { return this.Select.Offset; }
            set { this.Select.Offset = value; }
        }


        /// <summary>
        /// Devuelve la representacin SQL de la sentencia SELECT
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Clona la sentacia Select
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            SelectBuilder cloned = new SelectBuilder();

            cloned.select = (SelectClause) this.Select.Clone();
            cloned.from = (FromClause) this.From.Clone();
            cloned.where = (WhereClause) this.Where.Clone();
            cloned.groupBy = (GroupByClause) this.GroupBy.Clone();
            cloned.having = (HavingClause) this.Having.Clone();
            cloned.orderBy = (OrderByClause) this.OrderBy.Clone();
            return cloned;
        }

        #endregion
    }
}