using System.Configuration.Provider;
using System.Data;
using System.Data.Common;
using Avanzis.QueryFramework;

namespace Avanzis.QueryExecutor
{
    /// <summary>
    /// Clase base de la que heredarn todos los QueryExecutors
    /// </summary>
    public abstract class QueryFrameworkQueryFrameworkProvider : ProviderBase, IQueryFrameworkProviderFactory
    {
        #region IQueryExecutor Members

        /// <summary>
        /// Executes the command and returns a Dataset
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public abstract DataSet ExecuteDataset(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] parameters);

        /// <summary>
        /// Executes the dataset.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <returns></returns>
        public abstract DataSet ExecuteDataset(DbConnection connection, CommandType commandType, string commandText);

        /// <summary>
        /// Executes the command and returns a reader
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="externalConnection">if set to <c>true</c> [external connection].</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public abstract DbDataReader ExecuteReader(DbConnection connection, CommandType commandType, string commandText, bool externalConnection, params DbParameter[] parameters);

        /// <summary>
        /// Executes the reader.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="externalConnection">if set to <c>true</c> [external connection].</param>
        /// <returns></returns>
        public abstract DbDataReader ExecuteReader(DbConnection connection, CommandType commandType, string commandText, bool externalConnection);

        /// <summary>
        /// Executes the reader.
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public abstract DbDataReader ExecuteReader(DbTransaction transaction, CommandType commandType, string commandText, params DbParameter[] parameters);

        /// <summary>
        /// Executes a query but does not return anything
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        public abstract int ExecuteNonQuery(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] parameters);
        
        /// <summary>
        /// Executes the non query.
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public abstract int ExecuteNonQuery(DbTransaction transaction, CommandType commandType, string commandText, params DbParameter[] parameters);

        /// <summary>
        /// Executes the non query.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <returns></returns>
        public abstract int ExecuteNonQuery(DbConnection connection, CommandType commandType, string commandText);

        /// <summary>
        /// Executes the command that does not return
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <returns></returns>
        public abstract int ExecuteNonQuery(DbTransaction transaction, CommandType commandType, string commandText);        

        /// <summary>
        /// Executes the command and returns a scalar
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public abstract object ExecuteScalar(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] parameters);


        /// <summary>
        /// Executes the scalar.
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public abstract object ExecuteScalar(DbTransaction transaction, CommandType commandType, string commandText, params DbParameter[] parameters);
        /// <summary>
        /// Executes the scalar.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <returns></returns>
        public abstract object ExecuteScalar(DbConnection connection, CommandType commandType, string commandText);

        /// <summary>
        /// Executes the scalar.
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <returns></returns>
        public abstract object ExecuteScalar(DbTransaction transaction, CommandType commandType, string commandText);


        /// <summary>
        /// Processes the database value.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns></returns>
        public abstract object ProcessDatabaseValue(object value);

        #endregion

        #region DataProviderFactory 

        /// <summary>
        /// Gets a connection to the Database
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns></returns>
        public abstract DbConnection CreateConnection(string connectionString);

        public abstract DbCommand CreateCommand(string commandText, DbConnection connection);

        public abstract DbDataAdapter CreateDataAdapter(DbCommand command);

        public abstract DbDataAdapter CreateDataAdapter(string command, DbConnection connection);

        public abstract DbParameter CreateParameter(string parameterName, object parameterValue);
        
        #endregion

        #region Common Methods

        /// <summary>
        /// Executes command and returns a Dataset
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public DataSet ExecuteDataset(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters)
        {
            DataSet dataSet = null;

            using (DbConnection connection = CreateConnection(connectionString))
            {
                connection.Open();

                dataSet = ExecuteDataset(connection, commandType, commandText, parameters);

                connection.Close();
            }

            return dataSet;
        }

        /// <summary>
        /// Executes the non query.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        public int ExecuteNonQuery(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters)
        {
            int result = 0;

            using (DbConnection connection = CreateConnection(connectionString))
            {
                connection.Open();

                result = ExecuteNonQuery(connection, commandType, commandText, parameters);

                connection.Close();
            }

            return result;
        }

        /// <summary>
        /// Executes the command and returns a scalar object
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public object ExecuteScalar(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters)
        {
            object returnValue;

            using (DbConnection connection = CreateConnection(connectionString))
            {
                connection.Open();

                returnValue = ExecuteScalar(connection, commandType, commandText, parameters);

                connection.Close();
            }

            return returnValue;
        }

        /// <summary>
        /// Executes a commands and returns a reader
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public virtual DbDataReader ExecuteReader(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters)
        {
            //create & open a SqlConnection
            DbConnection connection = CreateConnection(connectionString);

            connection.Open();

            try
            {
                //call the private overload that takes an internally owned connection in place of the connection string
                return ExecuteReader(connection, commandType, commandText, false, parameters);                
            }
            catch
            {                
                connection.Close();
                throw;
            }
        }

        /// <summary>
        /// Executes the reader.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public virtual DbDataReader ExecuteReader(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] parameters)
        {
            return ExecuteReader(connection, commandType, commandText, true, parameters);
        }
      

        ///// <summary>
        ///// Executes the scalar.
        ///// </summary>
        ///// <param name="connectionString">The connection string.</param>
        ///// <param name="commandType">Type of the command.</param>
        ///// <param name="commandText">The command text.</param>
        ///// <param name="parameters">The parameters.</param>
        ///// <returns></returns>
        //public virtual T ExecuteScalar<T>(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters)
        //{
        //    return (T)ExecuteScalar(connectionString, commandType, commandText);
        //}

        ///// <summary>
        ///// Executes the scalar.
        ///// </summary>
        ///// <param name="connectionString">The connection string.</param>
        ///// <param name="commandType">Type of the command.</param>
        ///// <param name="commandText">The command text.</param>
        ///// <param name="parameters">The parameters.</param>
        ///// <returns></returns>
        //public virtual T ExecuteScalar<T>(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] parameters)
        //{
        //    Converter<
        //    return (T)ExecuteScalar(connection, commandType, commandText, parameters);
        //}

        /// <summary>
        /// Executes command and returns a Dataset
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public virtual DataTable ExecuteDataTable(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters)
        {
            DataSet dataSet = ExecuteDataset(connectionString, commandType, commandText, parameters);

            if (dataSet == null || dataSet.Tables.Count == 0)
                return null;
            else
                return dataSet.Tables[0];
        }

        /// <summary>
        /// Executes command and returns a Dataset
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public virtual DataTable ExecuteDataTable(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] parameters)
        {
            DataSet dataSet = ExecuteDataset(connection, commandType, commandText, parameters);

            if (dataSet == null || dataSet.Tables.Count == 0)
                return null;
            else
                return dataSet.Tables[0];
        }

        /// <summary>
        /// Builds a value parameter name for the current database.
        /// </summary>
        /// <param name="name">The name of the parameter.</param>
        /// <returns>A correctly formated parameter name.</returns>
        public virtual string BuildParameterName(string name)
        {
            return name;
        }

        // public abstract DbParameter CreateParameter(DbParameter parameter);          

        #endregion       

    }
}
