using System;
using System.Data.Common;
using Avanzis.QueryBuilder;
using Avanzis.QueryFramework;
using NUnit.Framework;

namespace Avanzis.QueryObjects.Tests
{
    [TestFixture]
    public class QueryObjectsBaseTest : BaseTest
    {
        [TestFixtureSetUp]
        public void SetUp()
        {
            ExecuteNonQuery("DELETE FROM REFEREES");
            
            for (int i = 0; i < 100; i++)
            {
                string sql = String.Format("INSERT INTO Referees (Code, Name, CountryCode) VALUES ('{0}','{1}','{2}')", i, i, i);
                ExecuteNonQuery(sql);
            }
        }


        [Test]
        public void TestSaveAssignedPrimaryKeyType()
        {
            int refereeCount = Convert.ToInt32(ExecuteScalar("select count(*) from referees where code = 'XXX'"));
            Assert.AreEqual(0, refereeCount);

            Referee r1 = new Referee();
            r1.Code = "XXX";
            r1.Name = "Referee XXX";
            r1.Save();

            // 
            refereeCount = Convert.ToInt32(ExecuteScalar("select count(*) from referees where code = 'XXX'"));
            Assert.AreEqual(1, refereeCount);

            Referee r2 = new Referee();
            r2.Code = "XXX";
            r2.Name = "Referre 2";
            r2.Save();

            // Carga el rbitro a partir del cdigo
            refereeCount = Convert.ToInt32(ExecuteScalar("select count(*) from referees where code = 'XXX'"));
            Assert.AreEqual(1, refereeCount);

            string refereeName = Convert.ToString(ExecuteScalar("select [name] from referees where code = 'XXX'"));
            Assert.AreEqual("Referre 2", refereeName);


            Referee r3 = new Referee();            
            r3.Load("XXX");            
            Assert.AreEqual("Referre 2", refereeName);

            ExecuteNonQuery("delete from referees where code = 'XXX'");
        }

        [Test]
        public void TestExists()
        {
            Referee r = new Referee();

            // Si no se ha guardado no puede existir
            Assert.AreEqual(false, r.Exists());

            r.Code = "XXX";

            Assert.AreEqual(false, r.Exists());

            r.Save();

            Assert.AreEqual(true, r.Exists());

            Referee r2 = new Referee();
            r2.Code = "XXX";
            Assert.AreEqual(true, r.Exists());

            ExecuteNonQuery("delete from referees where code = 'XXX'");
        }

        [Test]
        public void TestSaveIdentityPrimaryKeyType()
        {
            Group g1 = new Group();
            g1.Name = "X";
            g1.SeasonCode = "E2005";
            g1.PhaseTypeCode = "RS";

            Assert.Less(g1.GroupId, 1);

            g1.Save();

            Assert.Greater(g1.GroupId, 0);

            Group g2 = new Group();
            g2.Load(g1.GroupId);

            Assert.AreEqual(g1.Name, g2.Name);
            
            ExecuteNonQuery("delete from groups where Id = " + g1.GroupId);
        }

        [Test]
        public void TestLoadByFilters()
        {
            ExecuteNonQuery("DELETE FROM GROUPS WHERE Name = 'X' AND SeasonCode = 'E2005' AND PhaseTypeCode = 'RS'");

            Group g1 = new Group();
            g1.Name = "X";
            g1.SeasonCode = "E2005";
            g1.PhaseTypeCode = "RS";
            g1.Save();

            SearchFiltersCollection filters = new SearchFiltersCollection();            
            filters.Add("Name", SqlOperator.Equals, "X");
            filters.Add("SeasonCode", SqlOperator.Equals, "E2005");
            filters.Add("PhaseTypeCode", SqlOperator.Equals, "RS");

            Group g2 = new Group();
            g2.Load(filters);

            Assert.AreEqual(g1.GroupId, g2.GroupId);

            ExecuteNonQuery("DELETE FROM GROUPS WHERE Name = 'X' AND SeasonCode = 'E2005' AND PhaseTypeCode = 'RS'");
        }

        [Test]
        public void TestCount()
        {
            int databaseCount = Convert.ToInt32(ExecuteScalar("SELECT COUNT(*) FROM Referees"));

            int count = Referee.CountAll();

            Assert.AreEqual(databaseCount, count);

            databaseCount = Convert.ToInt32(ExecuteScalar("SELECT COUNT(*) FROM Referees WHERE Code Like 'A%'"));

            SearchFiltersCollection filters = new SearchFiltersCollection();
            filters.Add("Code Like 'A%'"); ;

            count = Referee.CountAll(filters);

            Assert.AreEqual(databaseCount, count);
        }

        [Test]
        public void TestDelete()
        {
            ExecuteNonQuery("delete from referees where code = 'XXX'");

            Referee r = new Referee();
            r.Code = "XXX";
            r.Save();

            int refereeCount = Convert.ToInt32(ExecuteScalar("select count(*) from referees where code = 'XXX'"));
            Assert.AreEqual(1, refereeCount);
            
            r.Delete();

            refereeCount = Convert.ToInt32(ExecuteScalar("select count(*) from referees where code = 'XXX'"));
            Assert.AreEqual(0, refereeCount);

            ExecuteNonQuery("delete from referees where code = 'XXX'");
        }

        [Test]
        public void TestFindAll()
        {
            int refereeCount = Convert.ToInt32(ExecuteScalar("select count(*) from referees"));

            Referee[] referees = Referee.FindAll();

            Assert.AreEqual(refereeCount, referees.Length);
        }

        [Test]
        public void TestFindAllWithOrder()
        {
            OrderItemsCollection orders = new OrderItemsCollection();
            orders.Add("Code", OrderDirection.Desc);

            Referee[] referees = Referee.FindAll(orders);

            string refereeCode = "ZZZ";

            foreach (Referee referee in referees)
            {
                bool olderIsGreaterOrEqual = (referee.Code.CompareTo(refereeCode) <= 0);
                Assert.IsTrue(olderIsGreaterOrEqual);
                refereeCode = referee.Code;
            }
        }

        [Test]
        public void TestFindAllWithFilter()
        {
            SearchFiltersCollection filters = new SearchFiltersCollection();
            filters.Add("Code > 'CCC'");
            filters.Add(FilterUnion.And, "Code < 'MMM'");

            int refereeCount = Convert.ToInt32(ExecuteScalar("select count(*) from referees where Code > 'CCC' AND code < 'MMM'"));
            
            Referee[] referees = Referee.FindAll(filters);

            Assert.AreEqual(refereeCount, referees.Length);            

            foreach (Referee referee in referees)
            {
                bool codeIsAllowedValue = (referee.Code.ToUpper().CompareTo("MMM") < 0) && (referee.Code.ToUpper().CompareTo("CCC") > 0);
                Assert.IsTrue(codeIsAllowedValue);                
            }
        }

        [Test]
        public void TestInheritanceWithSameTable()
        {
            DbDataReader reader = ExecuteReader("SELECT TOP 1 * FROM ClubGameStats");
            reader.Read();
            int id = Convert.ToInt32(reader["Id"]);
            int gameCode = Convert.ToInt32(reader["GameCode"]);
            string clubCode = Convert.ToString(reader["ClubCode"]);            
            string coacheCode = Convert.ToString(reader["CoachCode"]);
            reader.Close();
            
            ClubGameStats gameStats = new ClubGameStats();
            gameStats.Load(id);

            Assert.AreEqual(id, gameStats.Id);
            Assert.AreEqual(gameCode, gameStats.GameCode);
            Assert.AreEqual(clubCode, gameStats.ClubCode);            
            Assert.AreEqual(coacheCode, gameStats.CoachCode);
        }

        [Test]
        public void TestInheritanceWithDifferentTable()
        {
            Assert.AreEqual("ClubGameStats", ClubGameStats.GetSchema(typeof(ClubGameStats)).TableName);
            Assert.AreEqual("InheritedClubGameStats", ClubGameStats.GetSchema(typeof(InheritedClubGameStats)).TableName);
        }

        [Test]
        public void TestSlicedFindAllDialectMsSql2005Dialect()
        {
            QueryObjectsConfiguration.Dialect = Dialects.MSSql2005Dialect;
            QueryProcessor.ResetProcessor();

            OrderItemsCollection orders = new OrderItemsCollection("Code");

            Referee[] referres = Referee.FindAll(orders, null);

            Referee[] referres1to10 = Referee.SlicedFindAll(1, 10, orders, null);

            Assert.AreEqual(10, referres1to10.Length);

            Referee[] referees11to20 = Referee.SlicedFindAll(11, 10, orders, null);

            Assert.AreEqual(10, referres1to10.Length);

            for (int i = 0; i < 10; i++)
            {
                Assert.AreEqual(referres[i].Code, referres1to10[i].Code);
            }

            for (int i = 0; i < 10; i++)
            {
                Assert.AreEqual(referres[i + 10].Code, referees11to20[i].Code);
            }
        }

        [Test]
        public void TestSlicedFindAllDialectMsSql2000Dialect()
        {
            QueryObjectsConfiguration.Dialect = Dialects.MSSql2000Dialect;
            QueryProcessor.ResetProcessor();

            OrderItemsCollection orders = new OrderItemsCollection("Code");

            Referee[] referres = Referee.FindAll(orders, null);

            Referee[] referres1to10 = Referee.SlicedFindAll(1, 10, orders, null);

            Assert.AreEqual(10, referres1to10.Length);

            Referee[] referees11to20 = Referee.SlicedFindAll(11, 10, orders, null);

            Assert.AreEqual(10, referres1to10.Length);

            for (int i = 0; i < 10; i++)
            {
                Assert.AreEqual(referres[i].Code, referres1to10[i].Code);
            }

            for (int i = 0; i < 10; i++)
            {
                Assert.AreEqual(referres[i + 10].Code, referees11to20[i].Code);
            }
        }

    }
}
