using System;
using System.Collections.Generic;
using System.Reflection;

namespace Avanzis.QueryObjects
{
    /// <summary>
    /// Esquema de un objeto
    /// </summary>
    public class QueryObjectSchema
    {
        #region Variables

        private Type objectType;        
        private Dictionary<string,PersistentPropertyInfo> persistentProperties;
        private PrimaryKeyPropertyInfo primaryKey;
        private string tableName = null;

        #endregion

        #region Properties

        public Type ObjectType
        {
            get { return objectType; }
        }

        public Dictionary<string, PersistentPropertyInfo> PersistentProperties
        {
            get
            {
                return persistentProperties;
            }
        }

        public PrimaryKeyPropertyInfo PrimaryKey
        {
            get
            {
                return primaryKey;
            }
        }

        public bool HasPrimaryKey
        {
            get
            {
                return (primaryKey != null);
            }
        }

        /// <summary>
        /// Obtiene el nombre de la tabla donde se guardar este objeto.
        /// Si no est definido, supone que la tabla se llamar igual que el objeto
        /// </summary>
        /// <value>The name of the table.</value>
        public string TableName
        {
            get
            {                
                return tableName;
            }
        }

        #endregion

        #region Constructor

        private QueryObjectSchema(Type type)
        {
            this.objectType = type;
            UpdateSchema();
        }

        #endregion

        #region Methods

        private void UpdateSchema()
        {
            tableName = GetTypeDatabaseTable(objectType);
            FillPersistentPropertiesInfo();
        }

        /// <summary>
        /// Gets the type database table.
        /// </summary>
        /// <param name="type">The type.</param>
        /// <returns></returns>
        internal static String GetTypeDatabaseTable(Type type)
        {
            ActiveRecordAttribute persistentAttribute = GetTypePersistentAttribute(type);
            if (persistentAttribute == null) throw new QueryObjectsException("Type " + type.FullName + " has not an ActiveRecord attribute decorating the class");

            if (!string.IsNullOrEmpty(persistentAttribute.Table))
                return persistentAttribute.Table;
            else
                return type.Name;

        }

        /// <summary>
        /// Fills the persistent properties info.
        /// </summary>
        protected void FillPersistentPropertiesInfo()
        {
            persistentProperties = new Dictionary<string, PersistentPropertyInfo>();
            primaryKey = null;
            // Recorre el control aadiendo las propiedades que estn configuradas 			
            foreach (PropertyInfo propertyInfo in objectType.GetProperties(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance))
            {
                object[] propertyAttributes = propertyInfo.GetCustomAttributes(typeof(PropertyAttribute), true);
                if (propertyAttributes.Length > 0)
                {
                    PersistentPropertyInfo persistentPropertyInfo = new PersistentPropertyInfo(propertyInfo, (PropertyAttribute) propertyAttributes[0]);
                    persistentProperties.Add(persistentPropertyInfo.PropertyInfo.Name, persistentPropertyInfo);
                }
                if (primaryKey == null)
                {
                    object[] primaryKeyAttributes = propertyInfo.GetCustomAttributes(typeof(PrimaryKeyAttribute), true);
                    if (primaryKeyAttributes.Length > 0)
                    {
                        primaryKey = new PrimaryKeyPropertyInfo(propertyInfo, (PrimaryKeyAttribute)primaryKeyAttributes[0]);
                    }
                }
            }
        }

        /// <summary>
        /// Gets the type persistent attribute.
        /// </summary>
        /// <param name="type">The type.</param>
        /// <returns></returns>
        internal static ActiveRecordAttribute GetTypePersistentAttribute(Type type)
        {
            object[] persistentAttributes = type.GetCustomAttributes(typeof(ActiveRecordAttribute), true);
            if (persistentAttributes.Length == 1)
            {
                return (ActiveRecordAttribute)persistentAttributes[0];
            }
            else if (persistentAttributes.Length > 1)
            {
                throw new QueryObjectsException("An object can only have one PrimaryKey");
            }
            else
            {
                return null;
            }
        }  


        #endregion

        #region Static

        public static QueryObjectSchema GetSchema(Type objectType)
        {
            if (!Cache.ContainsKey(objectType))
            {
                QueryObjectSchema schema = new QueryObjectSchema(objectType);
                Cache[objectType] = schema;
            }
            return Cache[objectType];
            
        }

        #endregion

        #region Schema Cache

        // Singleton instance
        private static QueryObjectSchemaCache cache;

        // Lock synchronization object
        private static object syncLock = new object();

        /// <summary>
        /// Gets the current QueryProcessor based on the Dialect
        /// </summary>
        /// <value>The current.</value>
        internal static QueryObjectSchemaCache Cache 
        {
            get
            {
                if (cache == null)
                {
                    lock (syncLock)
                    {
                        if (cache == null)
                        {
                            cache = new QueryObjectSchemaCache();
                        }
                    }
                }

                return cache;
            }
        }

        /// <summary>
        /// Resets the processor.
        /// </summary>
        public static void ResetCache()
        {
            lock (syncLock)
            {
                cache = null;
            }
        }

        #endregion    

       
    }
}
