using System;
using System.Data;
using System.Data.Common;
using Avanzis.QueryExecutor;

namespace Avanzis.QueryObjects
{
    public class Session : IDisposable
    {
        #region variables

        private SessionFactory factory;
        private DbConnection connection;
        private DbTransaction transaction;
        private bool connectForNextOperation = false;
        private bool autoClose = false;

        #endregion

        #region Properties

        public DbConnection Connection
        {
            get
            {
                return connection;
            }
            set
            {
                connection = value;               
            }
        }

        public DbTransaction Transaction
        {
            get
            {
                return transaction;
            }
            set
            {
                transaction = value;
            }
        }

        public SessionFactory Factory
        {
            get
            {
                return factory;
            }
            set
            {
                factory = value;
            }
        }

        /// <summary>
        /// Is the <c>Session</c> still open?
        /// </summary>
        public bool IsOpen 
        { 
            get
            {
                if (connection == null)
                    return false;

                switch (connection.State)
                {
                    case ConnectionState.Open:
                        return true;
                    default:
                        return false;
                }
            }
        }

        public bool IsConnected
        {
            get { return connection != null || connectForNextOperation; }
        }

        public bool IsInActiveTransaction
        {
            get { return transaction != null; }
        }

        #endregion

        #region Constructor

        public Session()
        {

        }

        public Session(DbTransaction transaction)
        {
            this.Connection = transaction.Connection;
            this.Transaction = transaction;
        }

        public Session(DbConnection connection)
        {
            this.Connection = connection;
        }

        //public Session(DbConnection connection, SessionFactory factory, bool autoClose)
        //{
        //    this.Connection = connection;
        //    this.Factory = factory;
        //    this.autoClose = autoClose;
        //}

        #endregion

        #region Methods
        
        public void Open()
        {     
            // Crea la conexin
            Connect();
            // Si no estaba abierta, lo abrea
            if (!IsOpen)
                this.Connection.Open();
        }

        public void Connect()
        {
            if (connection == null)
            {
                connection = Database.CreateConnection(QueryObjectsConfiguration.ConnectionString);
                autoClose = true;
            }
            else if (connection.State == ConnectionState.Open)
            {
                autoClose = false;
            }
            else
            {
                autoClose = true;
            }

            connectForNextOperation = false;
        }

        public void Reconnect()
        {
            if (IsConnected)
            {
                throw new QueryObjectsException("session already connected");
            }

            connectForNextOperation = true;
            autoClose = true;
        }

        public DbConnection Close()
        {
            // When the connection is null nothing needs to be done - if there
            // is a value for connection then Disconnect() was not called - so we
            // need to ensure it gets called.
            if (connection == null)
            {
                connectForNextOperation = false;
                return null;
            }
            else
            {
                return Disconnect();
            }
        }

        public DbConnection Disconnect()
        {
            try
            {
                if (connectForNextOperation)
                {
                    connectForNextOperation = false;
                    return null;
                }
                else if (connection == null)
                {
                    throw new QueryObjectsException("Session already disconnected");
                }
                else
                {
                    DbConnection c = connection;

                    if (autoClose)
                    {
                        CloseConnection();
                        return null;
                    }
                    else
                    {
                        // Return the connection the user provided - at this point
                        // it has been dissociated from the NHibernate session.
                        connection = null;
                        return c;
                    }
                }
            }
            finally
            {
                // Ensure that AfterTransactionCompletion gets called since
                // it takes care of the locks and cache.
                if (!IsInActiveTransaction)
                {
                    // We don't know the state of the transaction
                    // session.AfterTransactionCompletion(false, null);
                }
            }
        }

        private void CloseConnection()
        {
            connection.Close();
            connection = null;
            connectForNextOperation = true;
        }

        public IDbConnection GetConnection()
        {
            if (connection == null)
            {
                if (connectForNextOperation)
                {
                    Connect();
                }
                else if (this.IsOpen)
                {
                    throw new QueryObjectsException("Session is currently disconnected");
                }
                else
                {
                    throw new QueryObjectsException("Session is closed");
                }
            }
            return connection;
        }

        #endregion

        #region Transactions

        public DbTransaction BeginTransaction(IsolationLevel isolationLevel)
        {
            this.transaction = this.Connection.BeginTransaction(isolationLevel);
            return this.Transaction;
        }

        public DbTransaction BeginTransaction()
        {
            if (!IsOpen) throw new QueryObjectsException("Session must be opened before beginning a transaction");

            this.transaction = this.Connection.BeginTransaction();
            return this.Transaction;
        }

        public void Commit()
        {
            if (transaction == null)
                throw new ArgumentNullException("transaction not defined");
            transaction.Commit();
            transaction.Dispose();
        }

        public void RollBack()
        {
            if (transaction == null)
                throw new ArgumentNullException("transaction not defined");
            transaction.Rollback();            
        }

        #endregion

        #region IDisposable Members

        public void Dispose()
        {
            if (transaction != null)
            {
                transaction.Dispose();
            }
            if (connection != null)
            {
                if (connection.State == ConnectionState.Closed)
                {
                    // log.Warn("finalizing unclosed session with closed connection");
                }
                else
                {
                    // if the Session is responsible for managing the connection then make sure
                    // the connection is disposed of.
                    if (autoClose)
                    {
                        CloseConnection();
                    }
                }
            }           
        }

        #endregion

        #region Conversiones

        public static implicit operator Session(DbConnection connection)
        {
            return new Session(connection);
        }

        public static implicit operator Session(DbTransaction transaction)
        {            
            Session session = new Session(transaction.Connection);
            session.Transaction = transaction;
            return session;
        }

        public static explicit operator DbConnection(Session session)
        {
            return session.Connection;
        }

        public static explicit operator DbTransaction(Session session)
        {
            return session.Transaction;
        }

        #endregion

        //public void Flush()
        //{
        //    // TODO
        //}

        //private FlushMode flushMode = FlushMode.Auto;

        ///// <summary>
        ///// Gets or sets the flush mode.
        ///// </summary>
        ///// <value>The flush mode.</value>
        //public FlushMode FlushMode
        //{
        //    get
        //    {
        //        return flushMode;
        //    }
        //    set
        //    {
        //        flushMode = value;
        //    }
        //}

        
    }

    //public enum FlushMode
    //{
    //    Auto,
    //    Never
    //}
}
